"""
SpamassassinChannels - command ``/bin/grep -r '^\\s*CHANNELURL=' /etc/mail/spamassassin/channel.d``
=========================================================================================================
"""
import re
from collections import OrderedDict

from insights.core import CommandParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.spamassassin_channels)
class SpamassassinChannels(CommandParser):
    """
    Class for parsing the ``/bin/grep -r '^\\s*CHANNELURL=' /etc/mail/spamassassin/channel.d``
    command.

    Attributes:
        channels (OrderedDict): channels grouped by configuration file

    Sample output of this command is::

        /etc/mail/spamassassin/channel.d/sought.conf:CHANNELURL=sought.rules.yerp.org
        /etc/mail/spamassassin/channel.d/spamassassin-official.conf:CHANNELURL=updates.spamassassin.org

    Examples:
        >>> type(spamassassin_channels)
        <class 'insights.parsers.spamassassin_channels.SpamassassinChannels'>
        >>> spamassassin_channels.channels
        OrderedDict([('/etc/mail/spamassassin/channel.d/sought.conf', ['sought.rules.yerp.org']), ('/etc/mail/spamassassin/channel.d/spamassassin-official.conf', ['updates.spamassassin.org'])])
    """
    def __init__(self, *args, **kwargs):
        self.channels = OrderedDict()
        super(SpamassassinChannels, self).__init__(*args, **kwargs)

    def parse_content(self, content):

        for line in content:
            file_name, file_line = line.split(":", 1)
            channel = re.sub('^\\s*CHANNELURL=', '', file_line).strip()
            if file_name not in self.channels:
                self.channels[file_name] = []
            self.channels[file_name].append(channel)



"""
SSSD Configuration - files ``/etc/sssd/sssd.conf`` and ``/etc/sssd.conf.d/*``
=============================================================================

SSSD configuration files are in INI format.

Example:
    [sssd]
    services = nss, sudo
    domains = test

    [sudo]
    debug_level = 0xfff0

    [domain/test]
    id_provider = ldap
    ldap_uri = ldap://ldap.test


SSSDConf - file ``/etc/sssd/sssd.conf``
----------------------------------------

SSSDConfd - files under ``/etc/sssd/conf.d/*.conf``
---------------------------------------------------
"""

from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.sssd_config)
class SSSDConf(IniConfigFile):
    """
    Parse the content of the ``/etc/sssd/sssd.conf``.

    The file format is standard ini file.
    """

    def getboolean(self, section, option):
        """
        Returns:
            bool: Returns boolean form based on the data from get.
        """
        val = self.get(section, option)
        boolean_states = {
            "true": True,
            "false": False,
        }

        if val.lower() not in boolean_states:
            raise ValueError("Not a boolean: %s" % val)

        return boolean_states[val.lower()]

    def _set(self, section, option, value=None):
        """
        Sets the value of the specified section option.

        Note: This should only be used by
        :class:`insights.combiners.sssd_conf.SSSDConfAll`.

        Args:
            section (str): The section str to set for. option (str): The option
            str to set for. value (str): The value to set.
        """
        section = section.strip()
        option = option.strip().lower()

        if section not in self._dict:
            self._dict[section] = {}

        self._dict[section][option] = value

    @property
    def domains(self):
        """
        Returns the list of domains defined in the 'sssd' section.  This is
        used to refer to the domain-specific sections of the configuration.
        """
        if self.has_option('sssd', 'domains'):
            domains = self.get('sssd', 'domains')
            if domains:
                return [domain.strip() for domain in domains.split(',')]

        # Return a blank list if no domains.
        return []

    def domain_config(self, domain):
        """
        Return the configuration dictionary for a specific domain, given as
        the raw name as listed in the 'domains' property of the sssd section.
        This then looks for the equivalent 'domain/{domain}' section of the
        config file.
        """
        full_domain = 'domain/' + domain
        if full_domain not in self:
            return {}
        return self.items(full_domain)


@parser(Specs.sssd_conf_d)
class SSSDConfd(SSSDConf):
    """
    Parse the content of configuration snippet under ``/etc/sssd/conf.d/*.conf``.

    The file format is standard ini file.
    """

    pass



#!/usr/bin/env python

import logging
import os
import tempfile

from contextlib import contextmanager

from insights.core.exceptions import InvalidContentType
from insights.util import fs, subproc, which
from insights.util.content_type import from_file as content_type_from_file

logger = logging.getLogger(__name__)


COMPRESSION_TYPES = ("zip", "tar", "gz", "bz2", "xz")


class ZipExtractor(object):
    def __init__(self, timeout=None):
        self.content_type = "application/zip"
        self.timeout = timeout
        self.tmp_dir = None
        self.created_tmp_dir = False

    def from_path(self, path, extract_dir=None, content_type=None):
        self.tmp_dir = tempfile.mkdtemp(prefix="insights-", dir=extract_dir)
        self.created_tmp_dir = True
        command = "unzip -n -q -d %s %s" % (self.tmp_dir, path)
        subproc.call(command, timeout=self.timeout)
        return self


class TarExtractor(object):

    def __init__(self, timeout=None):
        self.timeout = timeout
        self.tmp_dir = None
        self.created_tmp_dir = False

    TAR_FLAGS = {
        "application/x-xz": "-J",
        "application/x-gzip": "-I igzip" if which("igzip") else "-z",
        "application/gzip": "-I igzip" if which("igzip") else "-z",
        "application/x-bzip2": "-j",
        "application/x-tar": ""
    }

    def _tar_flag_for_content_type(self, content_type):
        flag = self.TAR_FLAGS.get(content_type)
        if flag is None:
            raise InvalidContentType(content_type)
        return flag

    def from_path(self, path, extract_dir=None, content_type=None):
        if os.path.isdir(path):
            self.tmp_dir = path
        else:
            self.content_type = content_type or content_type_from_file(path)
            tar_flag = self._tar_flag_for_content_type(self.content_type)
            self.tmp_dir = tempfile.mkdtemp(prefix="insights-", dir=extract_dir)
            self.created_tmp_dir = True
            command = "tar --delay-directory-restore %s -x --exclude=*/dev/* -f %s -C %s" % (tar_flag, path, self.tmp_dir)
            logging.debug("Extracting files in '%s'", self.tmp_dir)
            subproc.call(command, timeout=self.timeout)
        return self


class Extraction(object):
    def __init__(self, tmp_dir, content_type):
        self.tmp_dir = tmp_dir
        self.content_type = content_type


@contextmanager
def extract(path, timeout=None, extract_dir=None, content_type=None):
    """
    Extract path into a temporary directory in `extract_dir`.

    Yields an object containing the temporary path and the content type of the
    original archive.

    If the extraction takes longer than `timeout` seconds, the temporary path
    is removed, and an exception is raised.
    """
    content_type = content_type or content_type_from_file(path)
    if content_type == "application/zip":
        extractor = ZipExtractor(timeout=timeout)
    else:
        extractor = TarExtractor(timeout=timeout)

    try:
        ctx = extractor.from_path(path, extract_dir=extract_dir, content_type=content_type)
        content_type = extractor.content_type
        yield Extraction(ctx.tmp_dir, content_type)
    finally:
        if extractor.created_tmp_dir:
            fs.remove(extractor.tmp_dir, chmod=True)



"""
The serde module provides decorators that allow developers to register
serializer and deserializer functions for types. It also provides a
:py:class`Hydration` class that uses registered serde functions to save and
load objects from the file system. The Hydration class includes a
:py:func`Hydration.make_persister` method that returns a function appropriate
to register as an observer on a :py:class:`Broker`.
"""
import json as ser
import logging
import os
import time
import traceback

from glob import glob
from functools import partial

from insights.core import dr
from insights.core.exceptions import ContentException
from insights.util import fs

log = logging.getLogger(__name__)

SERIALIZERS = {}
DESERIALIZERS = {}


def serializer(_type):
    """
    Decorator for serializers.

    A serializer should accept two parameters: An object and a path which is
    a directory on the filesystem where supplementary data can be stored. This
    is most often useful for datasources. It should return a dictionary version
    of the original object that contains only elements that can be serialized
    to json.
    """

    def inner(func):
        name = dr.get_name(_type)
        if name in SERIALIZERS:
            msg = "%s already has a serializer registered: %s"
            raise Exception(msg % (name, dr.get_name(SERIALIZERS[name])))
        SERIALIZERS[name] = func
        return func
    return inner


def deserializer(_type):
    """
    Decorator for deserializers.

    A deserializer should accept three parameters: A type, a dictionary, and a
    path that may contain supplementary data stored by its paired serializer.
    If the serializer stores supplementary data, the relative path to it should
    be somewhere in the dict of the second parameter.
    """

    def inner(func):
        name = dr.get_name(_type)
        if name in DESERIALIZERS:
            msg = "%s already has a deserializer registered: %s"
            raise Exception(msg % (dr.get_name(name), dr.get_name(DESERIALIZERS[name])))
        DESERIALIZERS[name] = (_type, func)
        return func
    return inner


def get_serializer(obj):
    """ Get a registered serializer for the given object.

        This function walks the mro of obj looking for serializers.
        Returns None if no valid serializer is found.
    """
    return SERIALIZERS.get(dr.get_name(type(obj)))


def get_deserializer(obj):
    """ Returns a deserializer based on the fully qualified name string."""
    return DESERIALIZERS.get(dr.get_name(type(obj)))


def serialize(obj, root=None):
    to_dict = get_serializer(obj)
    return {
        "type": dr.get_name(type(obj)),
        "object": to_dict(obj, root=root),
    }


def deserialize(data, root=None, ctx=None, ds=None):
    type_data = DESERIALIZERS.get(data["type"])
    if type_data is None:
        raise Exception("Unrecognized type: %s" % data["type"])
    (_type, from_dict) = type_data
    return from_dict(_type, data["object"], root=root, ctx=ctx, ds=ds)


def marshal(comp, broker, root=None, pool=None):
    def call_serializer(func, value, exception_func):
        try:
            return func(value), None
        except Exception as ex:
            ex_tb = traceback.format_exc()
            exception_func(ex, ex_tb)
            return None, ex_tb

    def add_broker_exception(comp, broker, ex, ex_tb):
        broker.add_exception(comp, ex, ex_tb)

    v = broker.get(comp)
    if v is None:
        return None, None

    ser_func = partial(serialize, root=root)
    exc_func = partial(add_broker_exception, comp, broker)

    if isinstance(v, list):
        repeat = len(v)
        if pool:
            data = list(pool.map(call_serializer, [ser_func] * repeat, v, [exc_func] * repeat))
        else:
            data = list(map(call_serializer, [ser_func] * repeat, v, [exc_func] * repeat))
        results = [i[0] for i in data if i[0]]
        errors = [i[1] for i in data if i[1]]
        return results, errors
    return call_serializer(ser_func, v, exc_func)


def unmarshal(data, root=None, ctx=None, ds=None):
    if data is None:
        return
    if isinstance(data, list):
        return [deserialize(d, root=root, ctx=ctx, ds=ds) for d in data]
    return deserialize(data, root=root, ctx=ctx, ds=ds)


class Hydration(object):
    """
    The Hydration class is responsible for saving and loading insights
    components. It puts metadata about a component's evaluation in a metadata
    file for the component and allows the serializer for a component to put raw
    data beneath a working directory.
    """
    def __init__(self, root=None, ctx=None, meta_root="meta_data", data_root="data", pool=None):
        self.root = root
        self.ctx = ctx
        self.meta_root = os.path.join(root, meta_root) if root else None
        self.data_root = os.path.join(root, data_root) if root else None
        self.ser_name = dr.get_base_module_name(ser)
        self.created = False
        self.pool = pool

    def _hydrate_one(self, doc):
        """ Returns (component, results, errors, duration) """
        name = doc["name"]

        key = dr.get_component_by_name(name)
        if key is None:
            raise ValueError("{} is not a loaded component.".format(name))
        exec_time = doc["exec_time"]
        ser_time = doc["ser_time"]
        results = unmarshal(doc["results"], root=self.data_root, ctx=self.ctx, ds=key)
        return (key, results, exec_time, ser_time)

    def hydrate(self, broker=None):
        """
        Loads a Broker from a previously saved one. A Broker is created if one
        isn't provided.
        """

        broker = broker or dr.Broker()
        for path in glob(os.path.join(self.meta_root, "*")):
            try:
                with open(path) as f:
                    doc = ser.load(f)
                    res = self._hydrate_one(doc)
                    comp, results, exec_time, ser_time = res
                    if results:
                        broker[comp] = results
                        broker.exec_times[comp] = exec_time + ser_time
            except ContentException as ex:
                log.debug(ex)
            except ValueError as ve:
                log.debug(ve)
            except Exception as ex:
                log.warning(ex)
        return broker

    def dehydrate(self, comp, broker):
        """
        Saves a component in the given broker to the file system.
        """
        if not self.meta_root:
            raise Exception("Hydration meta_path not set. Can't dehydrate.")

        if not self.created:
            fs.ensure_path(self.meta_root, mode=0o770)
            if self.data_root:
                fs.ensure_path(self.data_root, mode=0o770)
            self.created = True

        try:
            name = dr.get_name(comp)

            # The `broker.tracebacks` is a dict in which the values are string
            # but not list of strings.
            errors = [broker.tracebacks[e] for e in broker.exceptions.get(comp, [])]

            start = time.time()
            results, ms_errors = marshal(comp, broker, root=self.data_root, pool=self.pool)
            errors.extend(ms_errors if isinstance(ms_errors, list) else [ms_errors]) if ms_errors else None

            doc = {
                "name": name,
                "exec_time": broker.exec_times.get(comp),
                "errors": errors,
                "results": results if results else None,
                "ser_time": time.time() - start
            }
        except Exception as ex:
            log.exception(ex)
        else:
            if doc is not None and (doc["results"] or doc["errors"]):
                path = None
                try:
                    path = os.path.join(self.meta_root, name + "." + self.ser_name)
                    with open(path, "w") as f:
                        ser.dump(doc, f)
                except Exception as boom:
                    log.error("Could not serialize %s to %s: %r" % (name, self.ser_name, boom))
                    if path:
                        fs.remove(path)

    def make_persister(self, to_persist):
        """
        Returns a function that hydrates components as they are evaluated. The
        function should be registered as an observer on a Broker just before
        execution.

        Args:
            to_persist (set): Set of components to persist. Skip everything
                else.
        """

        if not self.meta_root:
            raise Exception("Root not set. Can't create persister.")

        def persister(c, broker):
            if c in to_persist:
                self.dehydrate(c, broker)
        return persister



import pytest
import os
from io import TextIOWrapper, BytesIO
from insights.client.config import InsightsConfig, DEFAULT_OPTS
from mock.mock import patch, Mock
from pytest import mark


@patch('insights.client.config.ConfigParser.open')
def test_config_load(open_):
    open_.return_value = TextIOWrapper(BytesIO(b'[insights-client]\nusername=AMURO'))
    c = InsightsConfig()
    c._load_config_file()
    assert c.username == 'AMURO'


@patch('insights.client.config.ConfigParser.open')
def test_config_load_legacy(open_):
    open_.return_value = TextIOWrapper(BytesIO(b'[redhat-access-insights]\nusername=BRIGHT'))
    c = InsightsConfig()
    c._load_config_file()
    assert c.username == 'BRIGHT'


@patch('insights.client.config.ConfigParser.open')
def test_config_load_legacy_ignored(open_):
    open_.return_value = TextIOWrapper(
        BytesIO(b'[insights-client]\nusername=CASVAL\n' b'[redhat-access-insights]\nusername=SAYLA')
    )
    c = InsightsConfig()
    c._load_config_file()
    assert c.username == 'CASVAL'


@patch('insights.client.config.ConfigParser.open')
def test_config_load_section_error(open_):
    # defaults on incorrect conf
    open_.return_value = TextIOWrapper(BytesIO(b'aFUHAEFJhFhlAFJKhnfjeaf\nusername=RAMBA'))
    c = InsightsConfig()
    c._load_config_file()
    assert c.username == DEFAULT_OPTS['username']['default']


@patch('insights.client.config.ConfigParser.open')
def test_config_load_value_error(open_):
    # defaults on incorrect conf
    open_.return_value = TextIOWrapper(BytesIO(b'[insights-client]\nhttp_timeout=ZGOK'))
    c = InsightsConfig()
    c._load_config_file()
    assert c.http_timeout == DEFAULT_OPTS['http_timeout']['default']


def test_defaults():
    c = InsightsConfig()
    assert isinstance(c.cmd_timeout, int)
    assert isinstance(c.retries, int)
    assert isinstance(c.http_timeout, float)


@patch(
    'insights.client.config.os.environ',
    {'INSIGHTS_HTTP_TIMEOUT': '1234', 'INSIGHTS_RETRIES': '1234', 'INSIGHTS_CMD_TIMEOUT': '1234'},
)
def test_env_number_parsing():
    c = InsightsConfig()
    c._load_env()
    assert isinstance(c.cmd_timeout, int)
    assert isinstance(c.retries, int)
    assert isinstance(c.http_timeout, float)


@patch(
    'insights.client.config.os.environ',
    {
        'INSIGHTS_HTTP_TIMEOUT': 'STAY AWAY',
        'INSIGHTS_RETRIES': 'FROM ME',
        'INSIGHTS_CMD_TIMEOUT': 'BICK HAZARD',
    },
)
def test_env_number_bad_values():
    c = InsightsConfig()
    with pytest.raises(ValueError):
        c._load_env()


@patch('insights.client.config.os.environ', {})
def test_env_no_proxy_no_warning():
    with patch('insights.client.config.logger') as logger:
        c = InsightsConfig(_print_errors=True)
        c._load_env()
        logger.warning.assert_not_called()


@patch('insights.client.config.os.environ', {'HTTP_PROXY': '127.0.0.1'})
def test_env_http_proxy_warning():
    with patch('insights.client.config.logger') as logger:
        c = InsightsConfig(_print_errors=True)
        c._load_env()
        logger.warning.assert_called_once()


@patch('insights.client.config.os.environ', {'HTTP_PROXY': '127.0.0.1'})
@pytest.mark.parametrize(("kwargs",), (({},), ({"_print_errors": False},)))
def test_env_http_proxy_no_warning(kwargs):
    with patch('insights.client.config.logger') as logger:
        c = InsightsConfig(**kwargs)
        c._load_env()
        logger.warning.assert_not_called()


@patch('insights.client.config.os.environ', {'HTTP_PROXY': '127.0.0.1', 'HTTPS_PROXY': '127.0.0.1'})
def test_env_http_and_https_proxy_no_warning():
    with patch('insights.client.config.logger') as logger:
        c = InsightsConfig(_print_errors=True)
        c._load_env()
        logger.warning.assert_not_called()


@patch('insights.client.config.os.environ', {'HTTPS_PROXY': '127.0.0.1'})
def test_env_https_proxy_no_warning():
    with patch('insights.client.config.logger') as logger:
        c = InsightsConfig(_print_errors=True)
        c._load_env()
        logger.warning.assert_not_called()


@mark.parametrize(
    ("config",),
    (
        ({"payload": "./payload.tar.gz", "content_type": "application/gzip"},),
        ({"diagnosis": True},),
        ({"compliance": True},),
        ({"check_results": True},),
        ({"checkin": True},),
    ),
)
@patch('insights.client.config.argparse.ArgumentParser.parse_args')
def test_implied_non_legacy_upload(_, config):
    '''
    Some arguments should always imply legacy_upload=False.
    '''
    c = InsightsConfig(**config)
    c.load_all()
    assert c.legacy_upload is False


@patch('insights.client.config.argparse.ArgumentParser.parse_args')
def test_to_json_quiet_implies_diagnosis(_):
    '''
    --diagnosis should always imply legacy_upload=False
    '''
    c = InsightsConfig(to_json=True, quiet=True)
    c.load_all()
    assert c.diagnosis is True
    assert c.legacy_upload is False


def test_offline_disables_options():
    '''
    Can't use certain options in conjunction with --offline
    '''
    with pytest.raises(ValueError):
        InsightsConfig(to_json=True, offline=True)

    with pytest.raises(ValueError):
        InsightsConfig(test_connection=True, offline=True)

    with pytest.raises(ValueError):
        InsightsConfig(status=True, offline=True)

    with pytest.raises(ValueError):
        InsightsConfig(checkin=True, offline=True)

    with pytest.raises(ValueError):
        InsightsConfig(unregister=True, offline=True)


@patch('insights.client.config.argparse.ArgumentParser.parse_args')
def test_output_dir_file_cant_use_both(_):
    '''
    Cannot supply both --output-file and --output-dir
    '''
    with pytest.raises(ValueError):
        c = InsightsConfig(output_dir='test', output_file='test')
        c.load_all()


@patch('insights.client.config.argparse.ArgumentParser.parse_args')
def test_output_dir_file_validate(_):
    '''
    Must supply non-empty strings for --output-dir or --output-file
    '''
    with pytest.raises(ValueError):
        c = InsightsConfig(output_dir='')
        c.load_all()
    with pytest.raises(ValueError):
        c = InsightsConfig(output_file='')
        c.load_all()


@patch('insights.client.config.argparse.ArgumentParser.parse_args')
def test_output_dir_file_implies_no_upload_true_keep_archive_false(_):
    '''
    Using --output-dir or --tar-file should imply:
        no-upload    == True,  because we don't want to upload
        keep-archive == False, because we don't want to keep the files in temp
    '''
    c = InsightsConfig(output_dir='test')
    c.load_all()
    assert c.no_upload
    assert not c.keep_archive
    c = InsightsConfig(output_file='test')
    c.load_all()
    assert c.no_upload
    assert not c.keep_archive


@patch('insights.client.config.argparse.ArgumentParser.parse_args')
def test_compressor_option_validate(_):
    '''
    Compressor options are validated in config.py
    (used to be in archive.py)
    If an unsupported option is given, select 'gz'
    '''
    for comp in ('gz', 'bz2', 'xz', 'none'):
        c = InsightsConfig(compressor=comp)
        c.load_all()
        assert c.compressor == comp

    c = InsightsConfig(compressor='hullabaloo')
    c.load_all()
    assert c.compressor == 'gz'


@patch('insights.client.config.argparse.ArgumentParser.parse_args')
def test_output_file_guess_file_ext(_):
    '''
    If --output-file is selected, automatically guess
    the compressor option based on the file extension.

    If the compressor cannot be guessed from the filename,
    the filename will be given an extension based on the
    compressor option (default .tar.gz).

    If the proper extension is already part of the
    output file, the output file is unchanged.
    '''
    c = InsightsConfig(output_file='test-abc')
    c.load_all()
    assert c.output_file == os.path.abspath('test-abc.tar.gz')
    assert c.compressor == 'gz'

    c = InsightsConfig(output_file='test-def.tar.gz')
    c.load_all()
    assert c.output_file == os.path.abspath('test-def.tar.gz')
    assert c.compressor == 'gz'

    c = InsightsConfig(output_file='test-ghi.tar.bz2', compressor='bz2')
    c.load_all()
    assert c.output_file == os.path.abspath('test-ghi.tar.bz2')
    assert c.compressor == 'bz2'

    c = InsightsConfig(output_file='test-jkl', compressor='valkyrie')
    c.load_all()
    assert c.output_file == os.path.abspath('test-jkl.tar.gz')
    assert c.compressor == 'gz'

    c = InsightsConfig(output_file='test-mno.tar')
    c.load_all()
    assert c.output_file == os.path.abspath('test-mno.tar')
    assert c.compressor == 'none'


@patch('insights.client.config.argparse.ArgumentParser.parse_args', return_value=Mock(status=True))
def test_command_line_parse_twice(_):
    '''
    Verify that running _load_command_line() twice does not
    raise an argparse error.

    Previously would raise a SystemExit due to argparse not
    being loaded with the correct options.
    '''
    c = InsightsConfig()
    c._load_command_line()
    assert c.status
    c._load_command_line()
    assert c.status



"""
Modprobe configuration
======================

The modprobe configuration files are normally available to rules as a list of
ModProbe objects.  This combiner turns those into one set of data, preserving
the original file name that defined modprobe configuration line using a tuple.

"""

from insights.core.plugins import combiner
from insights.parsers.modprobe import ModProbe
from .. import LegacyItemAccess

from collections import namedtuple


ModProbeValue = namedtuple("ModProbeValue", ['value', 'source'])
"""
A value from a ModProbe source
"""


@combiner(ModProbe)
class AllModProbe(LegacyItemAccess):
    """
    Combiner for accessing all the modprobe configuration files in one
    structure.

    It's important for our reporting and information purposes to know not
    only what the configuration was but where it was defined.  Therefore, the
    format of the data in this combiner is slightly different compared to the
    ModProbe parser.  Here, each 'value' is actually a 2-tuple, with the
    actual data first and the file name from whence the value came second.
    This does mean that you need to pull the value out of each item - e.g.
    using a list comprehension - but it means that every item is associated
    with the file it was defined in.

    In line with the ModProbe configuration parser, the actual value is
    usually a list of the space-separated parts on the line, and the
    definitions for each module are similarly kept in a list, which makes

    Thanks to the LegacyItemAccess class, this can also be treated as a
    dictionary for look-ups of data in the `data` attribute.

    Attributes:
        data (dict): The combined data structures, with each item as a
            2-tuple, as described above.
        bad_lines(list): The list of unparseable lines from all files, with
            each line as a 2-tuple as described above.

    Sample data files::

        /etc/modprobe.conf:
            # watchdog drivers
            blacklist i8xx_tco

            # Don't install the Firewire ethernet driver
            install eth1394 /bin/true

        /etc/modprobe.conf.d/no_ipv6.conf:
            options ipv6 disable=1
            install ipv6 /bin/true

    Examples:
        >>> all_modprobe = shared[AllModProbe]
        >>> all_modprobe['alias']
        []
        >>> all_modprobe['blacklist']
        {'i8xx_tco': ModProbeValue(True, '/etc/modprobe.conf')}
        >>> all_modprobe['install']
        {'eth1394': ModProbeValue(['/bin/true'], '/etc/modprobe.conf'),
         'ipv6': ModProbeValue(['/bin/true'], '/etc/modprobe.conf.d/no_ipv6.conf')}
    """
    def __init__(self, modprobe):
        self.data = {}
        self.bad_lines = []
        for mod in modprobe:
            filename = mod.file_path  # relative path inside archive
            # Copy data section
            for section, sectdict in mod.data.items():
                if section not in self.data:
                    self.data[section] = {}
                for name, value in sectdict.items():
                    if name in self.data[section] and isinstance(self.data[section][name][0], list):
                        # append to this module's value - should only
                        # happen for aliases.
                        self.data[section][name][0].append(value)
                    else:
                        # create new tuple
                        self.data[section][name] = ModProbeValue(value=value, source=filename)
            # Copy bad lines, if any
            if mod.bad_lines:
                self.bad_lines.extend(
                    [ModProbeValue(value=line, source=filename) for line in mod.bad_lines]
                )
        super(AllModProbe, self).__init__()



from insights.combiners.grub_conf import GrubConf, BootLoaderEntries
from insights.combiners.grubby import Grubby, IsUEFIBoot
from insights.parsers.grub_conf import (
    Grub1Config,
    Grub2Config,
    Grub2EFIConfig,
    Grub1EFIConfig,
    BootLoaderEntries as BLE,
)
from insights.parsers.grubby import GrubbyDefaultIndex, GrubbyInfoAll
from insights.parsers.grubenv import GrubEnv
from insights.parsers.ls import LSlanFiltered
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.cmdline import CmdLine
from insights.tests import context_wrap
import pytest

GRUB1_TEMPLATE = """
# grub.conf generated by anaconda
#
# Note that you do not have to rerun grub after making changes to this file
# NOTICE:  You have a /boot partition.  This means that
#          all kernel and initrd paths are relative to /boot/, eg.
#          root (hd0,0)
#          kernel /vmlinuz-version ro root=/dev/mapper/VolGroup-lv_root
#          initrd /initrd-[generic-]version.img
#boot=/dev/sda
default=0
timeout=5
splashimage=(hd0,0)/grub/splash.xpm.gz
hiddenmenu
title Red Hat Enterprise Linux 6 (2.6.32-642.el6.x86_64)
        root (hd0,0)
        kernel /vmlinuz-2.6.32-642.el6.x86_64 {kernel_boot_options} ro root=/dev/mapper/VolGroup-lv_root intel_iommu=off rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet
        initrd /initramfs-2.6.32-642.el6.x86_64.img
title Red Hat Enterprise Linux 6 (2.6.32-642.el6.x86_64-2)
        root (hd0,0)
        kernel /vmlinuz-2.6.32-642.el6.x86_64 {kernel_boot_options} ro root=/dev/mapper/VolGroup-lv_root intel_iommu=on rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet
        initrd /initramfs-2.6.32-642.el6.x86_64.img
title Red Hat Enterprise Linux 6 (2.6.32-642.el6.x86_64-2)
        root (hd0,0)
        kernel /vmlinuz-2.6.32-642.el6.x86_64 {kernel_boot_options} ro root=/dev/mapper/VolGroup-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_NO_MD rd_LVM_LV=VolGroup/lv_swap SYSFONT=latarcyrheb-sun16 crashkernel=auto rd_LVM_LV=VolGroup/lv_root  KEYBOARDTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet
        initrd /initramfs-2.6.32-642.el6.x86_64.img
""".strip()  # noqa

# rhel-7
GRUB2_TEMPLATE = """
#
# DO NOT EDIT THIS FILE
#
# It is automatically generated by grub2-mkconfig using templates
# from /etc/grub.d and settings from /etc/default/grub
#

### BEGIN /etc/grub.d/00_header ###
set pager=1

terminal_output console
### END /etc/grub.d/00_header ###

### BEGIN /etc/grub.d/00_tuned ###
set tuned_params=""
### END /etc/grub.d/00_tuned ###

### BEGIN /etc/grub.d/01_users ###

### END /etc/grub.d/01_users ###

### BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server (3.10.0-327.el7.x86_64) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-327.el7.x86_64-advanced-4f80b3d4-90ba-4545-869c-febdecc586ce' {
        load_video
        set gfxpayload=keep
        insmod gzio
        insmod part_msdos
        insmod xfs
        set root='hd0,msdos1'
        if [ x$feature_platform_search_hint = xy ]; then
          search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  860a7b56-dbdd-498a-b085-53dc93e4650b
        else
          search --no-floppy --fs-uuid --set=root 860a7b56-dbdd-498a-b085-53dc93e4650b
        fi
        linux16 /vmlinuz-3.10.0-327.el7.x86_64 %s root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
        initrd16 /initramfs-3.10.0-327.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (0-rescue-9f20b35c9faa49aebe171f62a11b236f) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-9f20b35c9faa49aebe171f62a11b236f-advanced-4f80b3d4-90ba-4545-869c-febdecc586ce' {
        load_video
        insmod gzio
        insmod part_msdos
        insmod xfs
        set root='hd0,msdos1'
        if [ x$feature_platform_search_hint = xy ]; then
          search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  860a7b56-dbdd-498a-b085-53dc93e4650b
        else
          search --no-floppy --fs-uuid --set=root 860a7b56-dbdd-498a-b085-53dc93e4650b
        fi
        linux16 /vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f %s root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet
        initrd16 /initramfs-0-rescue-9f20b35c9faa49aebe171f62a11b236f.img
}
""".strip()  # noqa

GRUB2_TEMPLATE_BLSCFG = """
#
# DO NOT EDIT THIS FILE
#
# It is automatically generated by grub2-mkconfig using templates
# from /etc/grub.d and settings from /etc/default/grub
#

### BEGIN /etc/grub.d/00_header ###
set pager=1

terminal_output console
### END /etc/grub.d/00_header ###

### BEGIN /etc/grub.d/00_tuned ###
set tuned_params=""
### END /etc/grub.d/00_tuned ###

### BEGIN /etc/grub.d/01_users ###

### END /etc/grub.d/01_users ###
insmod blscfg
blscfg
if [ -s $prefix/grubenv ]; then
  load_env
fi

if [ -z "${kernelopts}" ]; then
  set kernelopts="root=/dev/mapper/rhel-root ro crashkernel=auto resume=/dev/mapper/rhel-swap rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet transparent_hugepage=never "
fi
""".strip()  # noqa

GRUB2_TEMPLATE_NO_BLSCFG = """
#
# DO NOT EDIT THIS FILE
#
# It is automatically generated by grub2-mkconfig using templates
# from /etc/grub.d and settings from /etc/default/grub
#

### BEGIN /etc/grub.d/00_header ###
set pager=1

terminal_output console
### END /etc/grub.d/00_header ###

### BEGIN /etc/grub.d/00_tuned ###
set tuned_params=""
### END /etc/grub.d/00_tuned ###

### BEGIN /etc/grub.d/01_users ###

### END /etc/grub.d/01_users ###
### BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server (4.18.0-240.el8.x86_64) 8.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-327.el7.x86_64-advanced-4f80b3d4-90ba-4545-869c-febdecc586ce' {
        load_video
        set gfxpayload=keep
        insmod gzio
        insmod part_msdos
        insmod xfs
        set root='hd0,msdos1'
        if [ x$feature_platform_search_hint = xy ]; then
          search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  860a7b56-dbdd-498a-b085-53dc93e4650b
        else
          search --no-floppy --fs-uuid --set=root 860a7b56-dbdd-498a-b085-53dc93e4650b
        fi
        linux16 /vmlinuz-4.18.0-240.el8.x86_64 %s root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
        initrd16 /initramfs-4.18.0-240.el8.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (0-rescue-9f20b35c9faa49aebe171f62a11b236f) 8.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-9f20b35c9faa49aebe171f62a11b236f-advanced-4f80b3d4-90ba-4545-869c-febdecc586ce' {
        load_video
        insmod gzio
        insmod part_msdos
        insmod xfs
        set root='hd0,msdos1'
        if [ x$feature_platform_search_hint = xy ]; then
          search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  860a7b56-dbdd-498a-b085-53dc93e4650b
        else
          search --no-floppy --fs-uuid --set=root 860a7b56-dbdd-498a-b085-53dc93e4650b
        fi
        linux16 /vmlinuz-0-rescue-9f20b35c9faa49aebe171f62a11b236f %s root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet
        initrd16 /initramfs-0-rescue-9f20b35c9faa49aebe171f62a11b236f.img
}
""".strip()  # noqa

GRUB2_EFI_CFG = """
### BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server (3.10.0-514.16.1.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {
    load_video
    set gfxpayload=keep
    insmod gzio
    insmod part_gpt
    insmod xfs
    set root='hd0,gpt2'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
    else
      search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
    fi
    linuxefi /vmlinuz-3.10.0-514.16.1.el7.x86_64 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
    initrdefi /initramfs-3.10.0-514.16.1.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (3.10.0-514.10.2.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {
    load_video
    set gfxpayload=keep
    insmod gzio
    insmod part_gpt
    insmod xfs
    set root='hd0,gpt2'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
    else
      search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
    fi
    linuxefi /vmlinuz-3.10.0-514.10.2.el7.x86_64 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
    initrdefi /initramfs-3.10.0-514.10.2.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (3.10.0-514.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {
    load_video
    set gfxpayload=keep
    insmod gzio
    insmod part_gpt
    insmod xfs
    set root='hd0,gpt2'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
    else
      search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
    fi
    linuxefi /vmlinuz-3.10.0-514.el7.x86_64 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
    initrdefi /initramfs-3.10.0-514.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (0-rescue-f1340b5dd6ee4c26b587621566111421) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-f1340b5dd6ee4c26b587621566111421-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {
    load_video
    insmod gzio
    insmod part_gpt
    insmod xfs
    set root='hd0,gpt2'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
    else
      search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
    fi
    linuxefi /vmlinuz-0-rescue-f1340b5dd6ee4c26b587621566111421 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet
    initrdefi /initramfs-0-rescue-f1340b5dd6ee4c26b587621566111421.img
}

### END /etc/grub.d/10_linux ###
""".strip()  # noqa

GRUB1_EFI_CFG = """
# grub.conf generated by anaconda
#
# Note that you do not have to rerun grub after making changes to this file
# NOTICE:  You have a /boot partition.  This means that
#          all kernel and initrd paths are relative to /boot/, eg.
#          root (hd0,1)
#          kernel /vmlinuz-version ro root=/dev/mapper/VolGroup-lv_root
#          initrd /initrd-[generic-]version.img
#boot=/dev/mpathap1
default=0
timeout=5
splashimage=(hd0,1)/grub/splash.xpm.gz
hiddenmenu
title Red Hat Enterprise Linux (2.6.32-71.el6.x86_64)
        root (hd0,1)
        kernel /vmlinuz-2.6.32-71.el6.x86_64 ro root=/dev/mapper/VolGroup-lv_root rd_LVM_LV=VolGroup/lv_root rd_LVM_LV=VolGroup/lv_swap rd_NO_LUKS rd_NO_MD rd_NO_DM LANG=en_US.UTF-8 SYSFONT=latarcyrheb-sun16 KEYBOARDTYPE=pc KEYTABLE=us crashkernel=auto rhgb quiet
        initrd /initramfs-2.6.32-71.el6.x86_64.img
""".strip()  # noqa

SYS_FIRMWARE_DIR_NOEFI = """
/sys/firmware:
total 0
drwxr-xr-x.  5 0 0 0 May 30 11:50 .
dr-xr-xr-x. 13 0 0 0 May 30 11:50 ..
drwxr-xr-x.  5 0 0 0 May 30 11:50 acpi
drwxr-xr-x.  3 0 0 0 May 30 11:51 dmi
drwxr-xr-x.  7 0 0 0 May 30 12:31 memmap

/sys/firmware/acpi:
total 0
drwxr-xr-x. 5 0 0    0 May 30 11:50 .
drwxr-xr-x. 5 0 0    0 May 30 11:50 ..
drwxr-xr-x. 6 0 0    0 May 30 12:31 hotplug
drwxr-xr-x. 2 0 0    0 May 30 12:31 interrupts
-r--r--r--. 1 0 0 4096 May 30 12:31 pm_profile
drwxr-xr-x. 3 0 0    0 May 30 11:50 tables
""".strip()

SYS_FIRMWARE_DIR_EFI = """
/sys/firmware:
total 0
drwxr-xr-x.  5 0 0 0 May 30 11:50 .
dr-xr-xr-x. 13 0 0 0 May 30 11:50 ..
drwxr-xr-x.  5 0 0 0 May 30 11:50 acpi
drwxr-xr-x.  3 0 0 0 May 30 11:51 dmi
drwxr-xr-x.  7 0 0 0 May 30 12:31 efi

/sys/firmware/efi:
total 0
drwxr-xr-x. 5 0 0    0 May 30 11:50 .
drwxr-xr-x. 5 0 0    0 May 30 11:50 ..
""".strip()

INSTALLED_RPMS_V1 = """
grub-0.97-94.el6.x86_64                     Mon Jan  8 18:35:25 2018
libreport-compat-2.0.9-24.el6.x86_64        Mon Jan  8 18:32:59 2018
make-3.81-20.el6.x86_64                     Mon Jan  8 18:31:49 2018
""".strip()

INSTALLED_RPMS_V2 = """
grub2-2.02-0.44.el7.x86_64                  Wed May 10 14:10:30 2017
libwbclient-4.4.4-12.el7_3.x86_64           Wed May 10 14:08:10 2017
xorg-x11-drv-vmmouse-13.0.0-12.el7.x86_64   Wed May 10 14:10:36 2017
""".strip()

CMDLINE_V1 = """
ro root=/dev/mapper/vg_rhel6box-lv_root rd_NO_LUKS LANG=en_US.UTF-8 rd_LVM_LV=vg_rhel6box/lv_swap rd_LVM_LV=vg_rhel6box/lv_root rd_NO_MD SYSFONT=latarcyrheb-sun16 crashkernel=129M@0M  KEYBOAR DTYPE=pc KEYTABLE=us rd_NO_DM rhgb quiet
""".strip()  # noqa

CMDLINE_V2 = """
BOOT_IMAGE=/vmlinuz-3.10.0-514.10.2.el7.x86_64 root=/dev/mapper/vg_system-lv_root ro crashkernel=auto rd.lvm.lv=vg_system/lv_root rd.lvm.lv=vg_system/lv_swap rhgb quiet LANG=en_US.UTF-8
""".strip()  # noqa

BOOT_LOADER_ENTRIES_1 = """
title Red Hat Enterprise Linux (4.18.0-80.1.2.el8_0.x86_64) 8.0 (Ootpa)
version 4.18.0-80.1.2.el8_0.x86_64
linux /vmlinuz-4.18.0-80.1.2.el8_0.x86_64
initrd /initramfs-4.18.0-80.1.2.el8_0.x86_64.img $tuned_initrd
options root=/dev/mapper/rhel_vm37--146-root ro crashkernel=auto resume=/dev/mapper/rhel_vm37--146-swap rd.lvm.lv=rhel_vm37-146/root rd.lvm.lv=rhel_vm37-146/swap $tuned_params noapic
id rhel-20190428101407-4.18.0-80.1.2.el8_0.x86_64
grub_users $grub_users
grub_arg --unrestricted
grub_class kernel
""".strip()  # noqa

BOOT_LOADER_ENTRIES_2 = """
title Red Hat Enterprise Linux (4.18.0-32.el8.x86_64) 8.0 (Ootpa)
version 4.18.0-32.el8.x86_64
linux /vmlinuz-4.18.0-32.el8.x86_64
initrd /initramfs-4.18.0-32.el8.x86_64.img
options root=/dev/mapper/rhel_rhel8-root ro elevator=noop no_timer_check crashkernel=auto resume=/dev/mapper/rhel_rhel8-swap rd.lvm.lv=rhel_rhel8/root rd.lvm.lv=rhel_rhel8/swap biosdevname=0 net.ifnames=0 rhgb
id rhel-20181027203430-4.18.0-32.el8.x86_64
grub_users $grub_users
grub_arg --unrestricted
grub_class kernel
""".strip()  # noqa

BOOT_LOADER_ENTRIES_3 = """
title Red Hat Enterprise Linux (4.18.0-305.el8.x86_64) 8.4 (Ootpa)
version 4.18.0-305.el8.x86_64
linux /vmlinuz-4.18.0-305.el8.x86_64
initrd /initramfs-4.18.0-305.el8.x86_64.img $tuned_initrd
options $kernelopts $tuned_params
id rhel-20210429130346-4.18.0-305.el8.x86_64
grub_users $grub_users
grub_arg --unrestricted
grub_class kernel
""".strip()

GRUBENV_WITH_TUNED_PARAMS = """
# GRUB Environment Block
saved_entry=295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64
kernelopts=root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200 noapic
boot_success=0
boot_indeterminate=2
tuned_params=transparent_hugepages=never
tuned_initrd=
###############################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################
""".strip()  # noqa

RHEL9_UEFI_HOST_GRUB_ENV = """
saved_entry=263d7f69235042b890394d6fe28ddeaf-5.14.0-570.30.1.el9_6.x86_64
menu_auto_hide=1
boot_success=1
boot_indeterminate=0
""".strip()

# RHEL9 UEFI host example: boot/efi/EFI/redhat/grub.cfg
RHEL9_UEFI_HOST_EFI_GRUB_CFG = """
search --no-floppy --fs-uuid --set=dev 5430c32f-3d13-477b-bb2b-13578b92bce8
set prefix=($dev)/grub2

export $prefix
configfile $prefix/grub.cfg
""".strip()

# RHEL9 UEFI host example: boot/grub2/grub.cfg
RHEL9_UEFI_HOST_GRUB2_CFG = """
#
# DO NOT EDIT THIS FILE
#
# It is automatically generated by grub2-mkconfig using templates
# from /etc/grub.d and settings from /etc/default/grub
#

### BEGIN /etc/grub.d/00_header ###
set pager=1

if [ -f ${config_directory}/grubenv ]; then
  load_env -f ${config_directory}/grubenv
elif [ -s $prefix/grubenv ]; then
  load_env
fi
if [ "${next_entry}" ] ; then
   set default="${next_entry}"
   set next_entry=
   save_env next_entry
   set boot_once=true
else
   set default="${saved_entry}"
fi

if [ x"${feature_menuentry_id}" = xy ]; then
  menuentry_id_option="--id"
else
  menuentry_id_option=""
fi

export menuentry_id_option

if [ "${prev_saved_entry}" ]; then
  set saved_entry="${prev_saved_entry}"
  save_env saved_entry
  set prev_saved_entry=
  save_env prev_saved_entry
  set boot_once=true
fi

function savedefault {
  if [ -z "${boot_once}" ]; then
    saved_entry="${chosen}"
    save_env saved_entry
  fi
}

function load_video {
  if [ x$feature_all_video_module = xy ]; then
    insmod all_video
  else
    insmod efi_gop
    insmod efi_uga
    insmod ieee1275_fb
    insmod vbe
    insmod vga
    insmod video_bochs
    insmod video_cirrus
  fi
}

terminal_output console
if [ x$feature_timeout_style = xy ] ; then
  set timeout_style=menu
  set timeout=5
# Fallback normal timeout code in case the timeout_style feature is
# unavailable.
else
  set timeout=5
fi
### END /etc/grub.d/00_header ###

### BEGIN /etc/grub.d/01_users ###
if [ -f ${prefix}/user.cfg ]; then
  source ${prefix}/user.cfg
  if [ -n "${GRUB2_PASSWORD}" ]; then
    set superusers="root"
    export superusers
    password_pbkdf2 ******** ${GRUB2_PASSWORD}
  fi
fi
### END /etc/grub.d/01_users ###

### BEGIN /etc/grub.d/08_fallback_counting ###
insmod increment
# Check if boot_counter exists and boot_success=0 to activate this behaviour.
if [ -n "${boot_counter}" -a "${boot_success}" = "0" ]; then
  # if countdown has ended, choose to boot rollback deployment,
  # i.e. default=1 on OSTree-based systems.
  if  [ "${boot_counter}" = "0" -o "${boot_counter}" = "-1" ]; then
    set default=1
    set boot_counter=-1
  # otherwise decrement boot_counter
  else
    decrement boot_counter
  fi
  save_env boot_counter
fi
### END /etc/grub.d/08_fallback_counting ###

### BEGIN /etc/grub.d/10_linux ###
insmod part_gpt
insmod xfs
search --no-floppy --fs-uuid --set=root 5430c32f-3d13-477b-bb2b-13578b92bce8
insmod part_gpt
insmod fat
search --no-floppy --fs-uuid --set=boot F900-6DBE

# This section was generated by a script. Do not modify the generated file - all changes
# will be lost the next time file is regenerated. Instead edit the BootLoaderSpec files.
#
# The blscfg command parses the BootLoaderSpec files stored in /boot/loader/entries and
# populates the boot menu. Please refer to the Boot Loader Specification documentation
# for the files format: https://systemd.io/BOOT_LOADER_SPECIFICATION/.

# The kernelopts variable should be defined in the grubenv file. But to ensure that menu
# entries populated from BootLoaderSpec files that use this variable work correctly even
# without a grubenv file, define a fallback kernelopts variable if this has not been set.
#
# The kernelopts variable in the grubenv file can be modified using the grubby tool or by
# executing the grub2-mkconfig tool. For the latter, the values of the GRUB_CMDLINE_LINUX
# and GRUB_CMDLINE_LINUX_DEFAULT options from /etc/default/grub file are used to set both
# the kernelopts variable in the grubenv file and the fallback kernelopts variable.
if [ -z "${kernelopts}" ]; then
  set kernelopts="root=/dev/mapper/rhel_vm--212--182-root ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel_vm--212--182-swap rd.lvm.lv=rhel_vm-212-182/root rd.lvm.lv=rhel_vm-212-182/swap "
fi

insmod blscfg
blscfg
### END /etc/grub.d/10_linux ###

### BEGIN /etc/grub.d/10_reset_boot_success ###
# Hiding the menu is ok if last boot was ok or if this is a first boot attempt to boot the entry
if [ "${boot_success}" = "1" -o "${boot_indeterminate}" = "1" ]; then
  set menu_hide_ok=1
else
  set menu_hide_ok=0
fi
# Reset boot_indeterminate after a successful boot
if [ "${boot_success}" = "1" ] ; then
  set boot_indeterminate=0
# Avoid boot_indeterminate causing the menu to be hidden more than once
elif [ "${boot_indeterminate}" = "1" ]; then
  set boot_indeterminate=2
fi
# Reset boot_success for current boot
set boot_success=0
save_env boot_success boot_indeterminate
### END /etc/grub.d/10_reset_boot_success ###

### BEGIN /etc/grub.d/12_menu_auto_hide ###
if [ x$feature_timeout_style = xy ] ; then
  if [ "${menu_show_once}" ]; then
    unset menu_show_once
    save_env menu_show_once
    set timeout_style=menu
    set timeout=60
  elif [ "${menu_auto_hide}" -a "${menu_hide_ok}" = "1" ]; then
    set orig_timeout_style=${timeout_style}
    set orig_timeout=${timeout}
    if [ "${fastboot}" = "1" ]; then
      # timeout_style=menu + timeout=0 avoids the countdown code keypress check
      set timeout_style=menu
      set timeout=0
    else
      set timeout_style=hidden
      set timeout=1
    fi
  fi
fi
### END /etc/grub.d/12_menu_auto_hide ###

### BEGIN /etc/grub.d/14_menu_show_once ###
if [ x$feature_timeout_style = xy ]; then
  if [ "${menu_show_once_timeout}" ]; then
    set timeout_style=menu
    set timeout="${menu_show_once_timeout}"
    unset menu_show_once_timeout
    save_env menu_show_once_timeout
  fi
fi
### END /etc/grub.d/14_menu_show_once ###

### BEGIN /etc/grub.d/20_linux_xen ###
### END /etc/grub.d/20_linux_xen ###

### BEGIN /etc/grub.d/20_ppc_terminfo ###
### END /etc/grub.d/20_ppc_terminfo ###

### BEGIN /etc/grub.d/30_os-prober ###
### END /etc/grub.d/30_os-prober ###

### BEGIN /etc/grub.d/30_uefi-firmware ###
if [ "$grub_platform" = "efi" ]; then
  menuentry 'UEFI Firmware Settings' $menuentry_id_option 'uefi-firmware' {
    fwsetup
  }
fi
### END /etc/grub.d/30_uefi-firmware ###

### BEGIN /etc/grub.d/35_fwupd ###
### END /etc/grub.d/35_fwupd ###

### BEGIN /etc/grub.d/40_custom ###
# This file provides an easy way to add custom menu entries.  Simply type the
# menu entries you want to add after this comment.  Be careful not to change
# the 'exec tail' line above.
### END /etc/grub.d/40_custom ###

### BEGIN /etc/grub.d/41_custom ###
if [ -f  ${config_directory}/custom.cfg ]; then
  source ${config_directory}/custom.cfg
elif [ -z "${config_directory}" -a -f  $prefix/custom.cfg ]; then
  source $prefix/custom.cfg
fi
### END /etc/grub.d/41_custom ###
"""

# boot/loader/entries/3b1bf67095e94696b600ed25416e97a8-5.14.0-503.11.1.el9_5.x86_64.conf
RHEL9_UEFI_HOST_BOOT_LOADER_ENTRY_0 = """
title Red Hat Enterprise Linux (5.14.0-503.11.1.el9_5.x86_64) 9.5 (Plow)
version 5.14.0-503.11.1.el9_5.x86_64
linux /vmlinuz-5.14.0-503.11.1.el9_5.x86_64
initrd /initramfs-5.14.0-503.11.1.el9_5.x86_64.img
options root=/dev/mapper/rhel_vm--212--182-root ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel_vm--212--182-swap rd.lvm.lv=rhel_vm-212-182/root rd.lvm.lv=rhel_vm-212-182/swap
grub_users $grub_users
grub_arg --unrestricted
grub_class rhel
""".strip()

# boot/loader/entries/3b1bf67095e94696b600ed25416e97a8-0-rescue.conf
RHEL9_UEFI_HOST_BOOT_LOADER_ENTRY_1 = """
title Red Hat Enterprise Linux (0-rescue-3b1bf67095e94696b600ed25416e97a8) 9.5 (Plow)
version 0-rescue-3b1bf67095e94696b600ed25416e97a8
linux /vmlinuz-0-rescue-3b1bf67095e94696b600ed25416e97a8
initrd /initramfs-0-rescue-3b1bf67095e94696b600ed25416e97a8.img
options root=/dev/mapper/rhel_vm--212--182-root ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel_vm--212--182-swap rd.lvm.lv=rhel_vm-212-182/root rd.lvm.lv=rhel_vm-212-182/swap
grub_users $grub_users
grub_arg --unrestricted
grub_class rhel
""".strip()

RHEL9_UEFI_HOST_LS_SYS_FIRMWARE_EFI = """
/sys/firmware:
total 0
drwxr-xr-x  6 0 0 0 Aug 19 14:57 efi
""".strip()

RHEL9_UEFI_HOST_GRUBBY_DEFAULT_INDEX = """
0
""".strip()

RHEL9_UEFI_HOST_GRUBBY_INFO_ALL = """
index=0
kernel="/boot/vmlinuz-5.14.0-503.11.1.el9_5.x86_64"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel_vm--212--182-swap rd.lvm.lv=rhel_vm-212-182/root rd.lvm.lv=rhel_vm-212-182/swap"
root="/dev/mapper/rhel_vm--212--182-root"
initrd="/boot/initramfs-5.14.0-503.11.1.el9_5.x86_64.img"
title="Red Hat Enterprise Linux (5.14.0-503.11.1.el9_5.x86_64) 9.5 (Plow)"
id="3b1bf67095e94696b600ed25416e97a8-5.14.0-503.11.1.el9_5.x86_64"
index=1
kernel="/boot/vmlinuz-0-rescue-3b1bf67095e94696b600ed25416e97a8"
args="ro crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M resume=/dev/mapper/rhel_vm--212--182-swap rd.lvm.lv=rhel_vm-212-182/root rd.lvm.lv=rhel_vm-212-182/swap"
root="/dev/mapper/rhel_vm--212--182-root"
initrd="/boot/initramfs-0-rescue-3b1bf67095e94696b600ed25416e97a8.img"
title="Red Hat Enterprise Linux (0-rescue-3b1bf67095e94696b600ed25416e97a8) 9.5 (Plow)"
id="3b1bf67095e94696b600ed25416e97a8-0-rescue
""".strip()


def test_grub1_only1():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    cmdline = CmdLine(context_wrap(CMDLINE_V1))
    result = GrubConf(None, grub1, None, None, None, None, None, cmdline, IsUEFIBoot(None, None))
    assert result.kernel_initrds['grub_kernels'][0] == 'vmlinuz-2.6.32-642.el6.x86_64'
    assert result.kernel_initrds['grub_initrds'][0] == 'initramfs-2.6.32-642.el6.x86_64.img'
    assert result.is_kdump_iommu_enabled is True
    assert result.get_grub_cmdlines() == result.get_grub_cmdlines('/vmlinuz')
    assert len(result.get_grub_cmdlines()) == 3
    assert result.version == 1
    assert result.is_efi is False


def test_grub1_cmdline():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub1e = Grub1EFIConfig(context_wrap(GRUB1_EFI_CFG))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    cmdline = CmdLine(context_wrap(CMDLINE_V1))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_NOEFI))
    result = GrubConf(
        None, grub1, grub2, grub1e, grub2e, None, None, cmdline, IsUEFIBoot(ls_lan, None)
    )
    assert result.kernel_initrds['grub_kernels'][0] == 'vmlinuz-2.6.32-642.el6.x86_64'
    assert result.kernel_initrds['grub_initrds'][0] == 'initramfs-2.6.32-642.el6.x86_64.img'
    assert result.is_kdump_iommu_enabled is True
    assert result.get_grub_cmdlines() == result.get_grub_cmdlines('/vmlinuz')
    assert len(result.get_grub_cmdlines()) == 3
    assert result.version == 1
    assert result.is_efi is False


def test_grub1_efi_cmdline():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub1e = Grub1EFIConfig(context_wrap(GRUB1_EFI_CFG))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    cmdline = CmdLine(context_wrap(CMDLINE_V1))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_EFI))
    result = GrubConf(
        None, grub1, grub2, grub1e, grub2e, None, None, cmdline, IsUEFIBoot(ls_lan, None)
    )
    assert result.kernel_initrds['grub_kernels'][0] == 'vmlinuz-2.6.32-71.el6.x86_64'
    assert result.kernel_initrds['grub_initrds'][0] == 'initramfs-2.6.32-71.el6.x86_64.img'
    assert result.is_kdump_iommu_enabled is False
    assert len(result.get_grub_cmdlines()) == 1
    assert result.version == 1
    assert result.is_efi is True


def test_grub1_rpms():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub1e = Grub1EFIConfig(context_wrap(GRUB1_EFI_CFG))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V1))
    cmdline = CmdLine(context_wrap(CMDLINE_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_NOEFI))
    result = GrubConf(
        None, grub1, grub2, grub1e, grub2e, None, rpms, cmdline, IsUEFIBoot(ls_lan, None)
    )
    assert result.kernel_initrds['grub_kernels'][0] == 'vmlinuz-2.6.32-642.el6.x86_64'
    assert result.kernel_initrds['grub_initrds'][0] == 'initramfs-2.6.32-642.el6.x86_64.img'
    assert result.is_kdump_iommu_enabled is True
    assert result.get_grub_cmdlines() == result.get_grub_cmdlines('/vmlinuz')
    assert len(result.get_grub_cmdlines()) == 3
    assert result.version == 1
    assert result.is_efi is False


def test_grub1_efi_rpms():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub1e = Grub1EFIConfig(context_wrap(GRUB1_EFI_CFG))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V1))
    cmdline = CmdLine(context_wrap(CMDLINE_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_EFI))
    result = GrubConf(
        None, grub1, grub2, grub1e, grub2e, None, rpms, cmdline, IsUEFIBoot(ls_lan, None)
    )
    assert result.kernel_initrds['grub_kernels'][0] == 'vmlinuz-2.6.32-71.el6.x86_64'
    assert result.kernel_initrds['grub_initrds'][0] == 'initramfs-2.6.32-71.el6.x86_64.img'
    assert result.is_kdump_iommu_enabled is False
    assert result.get_grub_cmdlines() == result.get_grub_cmdlines('/vmlinuz')
    assert len(result.get_grub_cmdlines()) == 1
    assert result.version == 1
    assert result.is_efi is True


def test_grub2_cmdline():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub1e = Grub1EFIConfig(context_wrap(GRUB1_EFI_CFG))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    cmdline = CmdLine(context_wrap(CMDLINE_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_NOEFI))
    result = GrubConf(
        None, grub1, grub2, grub1e, grub2e, None, None, cmdline, IsUEFIBoot(ls_lan, None)
    )
    assert result.kernel_initrds['grub_kernels'][0] == 'vmlinuz-3.10.0-327.el7.x86_64'
    assert result.kernel_initrds['grub_initrds'][0] == 'initramfs-3.10.0-327.el7.x86_64.img'
    assert result.is_kdump_iommu_enabled is False
    assert (
        result.get_grub_cmdlines('/vmlinuz-3.10.0')[0].name
        == "'Red Hat Enterprise Linux Server (3.10.0-327.el7.x86_64) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-327.el7.x86_64-advanced-4f80b3d4-90ba-4545-869c-febdecc586ce'"
    )  # noqa
    assert result.get_grub_cmdlines('test') == []
    assert result.get_grub_cmdlines('') == []
    assert len(result.get_grub_cmdlines()) == 2
    assert result.version == 2
    assert result.is_efi is False


def test_grub2_efi_cmdline():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub1e = Grub1EFIConfig(context_wrap(GRUB1_EFI_CFG))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    cmdline = CmdLine(context_wrap(CMDLINE_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_EFI))
    result = GrubConf(
        None, grub1, grub2, grub1e, grub2e, None, None, cmdline, IsUEFIBoot(ls_lan, None)
    )
    assert result.get_grub_cmdlines() == result.get_grub_cmdlines('/vmlinuz')
    assert result.get_grub_cmdlines('rescue')[0].name.startswith(
        "'Red Hat Enterprise Linux Server (0-rescue"
    )
    assert len(result.get_grub_cmdlines()) == 4
    assert result.version == 2
    assert result.is_efi is True


def test_grub2_rpms():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub1e = Grub1EFIConfig(context_wrap(GRUB1_EFI_CFG))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V2))
    cmdline = CmdLine(context_wrap(CMDLINE_V1))
    result = GrubConf(
        None, grub1, grub2, grub1e, grub2e, None, rpms, cmdline, IsUEFIBoot(None, None)
    )
    assert result.kernel_initrds['grub_kernels'][0] == 'vmlinuz-3.10.0-327.el7.x86_64'
    assert result.kernel_initrds['grub_initrds'][0] == 'initramfs-3.10.0-327.el7.x86_64.img'
    assert result.is_kdump_iommu_enabled is False
    assert (
        result.get_grub_cmdlines('/vmlinuz-3.10.0')[0].name
        == "'Red Hat Enterprise Linux Server (3.10.0-327.el7.x86_64) 7.2 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-327.el7.x86_64-advanced-4f80b3d4-90ba-4545-869c-febdecc586ce'"
    )  # noqa
    assert result.get_grub_cmdlines('test') == []
    assert result.get_grub_cmdlines('') == []
    assert len(result.get_grub_cmdlines()) == 2
    assert result.version == 2
    assert result.is_efi is False


def test_grub2_efi_rpms():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub1e = Grub1EFIConfig(context_wrap(GRUB1_EFI_CFG))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V2))
    cmdline = CmdLine(context_wrap(CMDLINE_V1))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_EFI))
    result = GrubConf(
        None, grub1, grub2, grub1e, grub2e, None, rpms, cmdline, IsUEFIBoot(ls_lan, None)
    )
    assert result.kernel_initrds['grub_initrds'][0] == 'initramfs-3.10.0-514.16.1.el7.x86_64.img'
    assert result.get_grub_cmdlines() == result.get_grub_cmdlines('/vmlinuz')
    assert result.get_grub_cmdlines('rescue')[0].name.startswith(
        "'Red Hat Enterprise Linux Server (0-rescue"
    )
    assert len(result.get_grub_cmdlines()) == 4
    assert result.version == 2
    assert result.is_efi is True


def test_get_grub_cmdlines_none():
    grub1 = Grub1Config(context_wrap(GRUB1_TEMPLATE))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    cmdline = CmdLine(context_wrap(CMDLINE_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_EFI))
    with pytest.raises(Exception) as pe:
        GrubConf(None, grub1, grub2, None, None, None, None, cmdline, IsUEFIBoot(ls_lan, None))
    assert "No valid grub configuration is found." in str(pe.value)

    grub1e = Grub1EFIConfig(context_wrap(GRUB1_TEMPLATE))
    grub2e = Grub2EFIConfig(context_wrap(GRUB2_TEMPLATE))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V2))
    with pytest.raises(Exception) as pe:
        GrubConf(None, None, None, grub1e, grub2e, None, rpms, None, IsUEFIBoot(None, None))
    assert "No valid grub configuration is found." in str(pe.value)

    grub2e = Grub2EFIConfig(context_wrap(GRUB2_EFI_CFG))
    with pytest.raises(Exception) as pe:
        GrubConf(None, grub1, None, grub1e, grub2e, None, rpms, None, IsUEFIBoot(None, None))
    assert "No valid grub configuration is found." in str(pe.value)


def test_grub2_grubenv():
    grubenv = GrubEnv(context_wrap(GRUBENV_WITH_TUNED_PARAMS))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE))
    grub_ble1 = BLE(context_wrap(BOOT_LOADER_ENTRIES_1))
    grub_ble2 = BLE(context_wrap(BOOT_LOADER_ENTRIES_2))
    grub_bles = BootLoaderEntries([grub_ble1, grub_ble2], grubenv, IsUEFIBoot(None, None))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_NOEFI))
    result = GrubConf(
        None, None, grub2, None, None, grub_bles, rpms, None, IsUEFIBoot(ls_lan, None)
    )
    assert len(result.get_grub_cmdlines()) == 2
    assert 'noapic' not in result.get_grub_cmdlines()[1]['cmdline']
    assert 'transparent_hugepages' not in result.get_grub_cmdlines()[0]['cmdline']
    assert result.version == 2
    assert not result.is_efi


def test_grub2_grubenv_with_kernelopts():
    grubenv = GrubEnv(context_wrap(GRUBENV_WITH_TUNED_PARAMS))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE_BLSCFG))
    grub_ble1 = BLE(context_wrap(BOOT_LOADER_ENTRIES_1))
    grub_ble2 = BLE(context_wrap(BOOT_LOADER_ENTRIES_2))
    grub_ble3 = BLE(context_wrap(BOOT_LOADER_ENTRIES_3))
    grub_bles = BootLoaderEntries(
        [grub_ble1, grub_ble2, grub_ble3], grubenv, IsUEFIBoot(None, None)
    )
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_NOEFI))
    result = GrubConf(
        None, None, grub2, None, None, grub_bles, rpms, None, IsUEFIBoot(ls_lan, None)
    )
    assert len(result.get_grub_cmdlines()) == 3
    assert 'noapic' in result.get_grub_cmdlines()[2]['cmdline']
    assert 'transparent_hugepages' in result.get_grub_cmdlines()[2]['cmdline']
    assert result.version == 2
    assert not result.is_efi


def test_grub2_with_blscfg():
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE_BLSCFG))
    grub_ble1 = BLE(context_wrap(BOOT_LOADER_ENTRIES_1))
    grub_ble2 = BLE(context_wrap(BOOT_LOADER_ENTRIES_2))
    grub_ble3 = BLE(context_wrap(BOOT_LOADER_ENTRIES_3))
    grub_bles = BootLoaderEntries([grub_ble1, grub_ble2, grub_ble3], None, IsUEFIBoot(None, None))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_NOEFI))
    result = GrubConf(
        None, None, grub2, None, None, grub_bles, rpms, None, IsUEFIBoot(ls_lan, None)
    )
    assert len(result.get_grub_cmdlines()) == 3
    assert 'noapic' in result.get_grub_cmdlines()[0]['cmdline']
    assert 'transparent_hugepages' not in result.get_grub_cmdlines()[0]['cmdline']
    assert result.version == 2
    assert not result.is_efi


def test_grub2_boot_loader_entries():
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE_BLSCFG))
    grub_ble1 = BLE(context_wrap(BOOT_LOADER_ENTRIES_1))
    grub_ble2 = BLE(context_wrap(BOOT_LOADER_ENTRIES_2))
    grub_bles = BootLoaderEntries([grub_ble1, grub_ble2], None, IsUEFIBoot(None, None))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_NOEFI))
    result = GrubConf(
        None, None, grub2, None, None, grub_bles, rpms, None, IsUEFIBoot(ls_lan, None)
    )
    assert len(result.get_grub_cmdlines()) == 2
    assert 'noapic' in result.get_grub_cmdlines()[0]['cmdline']
    assert result.version == 2
    assert not result.is_efi


def test_grub2_boot_loader_entries_with_grubenv():
    grubenv = GrubEnv(context_wrap(GRUBENV_WITH_TUNED_PARAMS))
    grub2 = Grub2Config(context_wrap(GRUB2_TEMPLATE_BLSCFG))
    grub_ble1 = BLE(context_wrap(BOOT_LOADER_ENTRIES_1))
    grub_ble3 = BLE(context_wrap(BOOT_LOADER_ENTRIES_3))
    grub_bles = BootLoaderEntries([grub_ble1, grub_ble3], grubenv, IsUEFIBoot(None, None))
    rpms = InstalledRpms(context_wrap(INSTALLED_RPMS_V2))
    ls_lan = LSlanFiltered(context_wrap(SYS_FIRMWARE_DIR_NOEFI))
    result = GrubConf(
        None, None, grub2, None, None, grub_bles, rpms, None, IsUEFIBoot(ls_lan, None)
    )
    assert len(result.get_grub_cmdlines()) == 2
    assert 'noapic' in result.get_grub_cmdlines()[0]['cmdline']
    assert 'transparent_hugepages' in result.get_grub_cmdlines()[0]['cmdline']
    assert 'noapic' in result.get_grub_cmdlines()[1]['cmdline']
    assert 'transparent_hugepages' in result.get_grub_cmdlines()[1]['cmdline']
    assert result.version == 2
    assert not result.is_efi


def test_grub2_rhel9_efi_grubby():
    grubenv = GrubEnv(context_wrap(RHEL9_UEFI_HOST_GRUB_ENV))
    grub2 = Grub2Config(context_wrap(RHEL9_UEFI_HOST_GRUB2_CFG))
    grub2_efi = Grub2EFIConfig(context_wrap(RHEL9_UEFI_HOST_EFI_GRUB_CFG))
    grub_ble0 = BLE(context_wrap(RHEL9_UEFI_HOST_BOOT_LOADER_ENTRY_0))
    grub_ble1 = BLE(context_wrap(RHEL9_UEFI_HOST_BOOT_LOADER_ENTRY_1))
    grub_bles = BootLoaderEntries([grub_ble0, grub_ble1], grubenv, IsUEFIBoot(None, None))
    ls_lan = LSlanFiltered(context_wrap(RHEL9_UEFI_HOST_LS_SYS_FIRMWARE_EFI))

    grubby_info_all = GrubbyInfoAll(context_wrap(RHEL9_UEFI_HOST_GRUBBY_INFO_ALL))
    grubby_default_index = GrubbyDefaultIndex(context_wrap(RHEL9_UEFI_HOST_GRUBBY_DEFAULT_INDEX))
    grubby = Grubby(grubby_info_all, grubby_default_index, grubenv, IsUEFIBoot(ls_lan, None))

    result = GrubConf(
        grubby, None, grub2, None, grub2_efi, grub_bles, None, None, IsUEFIBoot(ls_lan, None)
    )
    assert len(result.get_grub_cmdlines()) == 2
    assert 'root=/dev/mapper/rhel_vm--212--182-root' in result.get_grub_cmdlines()[0]['cmdline']
    assert (
        'crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M' in result.get_grub_cmdlines()[1]['cmdline']
    )
    assert result.version == 2
    assert result.is_efi



"""
IPTables configuration
======================

Module for processing output of the ``iptables-save`` and ``ip6tables-save``
commands.  Parsers included are:

IPTables - command ``iptables-save``
------------------------------------

IP6Tables - command ``ip6tables-save``
--------------------------------------

IPTabPermanent - file ``/etc/sysconfig/iptables``
-------------------------------------------------

IP6TabPermanent - file ``/etc/sysconfig/ip6tables``
---------------------------------------------------

Sample input data looks like::

    # Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
    *filter
    :INPUT ACCEPT [0:0]
    :FORWARD ACCEPT [0:0]
    :OUTPUT ACCEPT [769:196899]
    :REJECT-LOG - [0:0]
    -A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
    -A INPUT -s 192.168.0.0/24 -j ACCEPT
    -A INPUT -p icmp -j ACCEPT
    -A INPUT -p tcp -m state --state NEW -m tcp --dport 22 -j ACCEPT
    -A INPUT -j REJECT --reject-with icmp-host-prohibited
    -A REJECT-LOG -p tcp -j REJECT --reject-with tcp-reset
    COMMIT
    # Completed on Tue Aug 16 10:18:43 2016
    # Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
    *mangle
    :PREROUTING ACCEPT [451:22060]
    :INPUT ACCEPT [451:22060]
    :FORWARD ACCEPT [0:0]
    :OUTPUT ACCEPT [594:47151]
    :POSTROUTING ACCEPT [594:47151]
    COMMIT
    # Completed on Tue Aug 16 10:18:43 2016
    # Generated by iptables-save v1.4.7 on Tue Aug 16 10:18:43 2016
    *nat
    :PREROUTING ACCEPT [0:0]
    :POSTROUTING ACCEPT [3:450]
    :OUTPUT ACCEPT [3:450]
    COMMIT
    # Completed on Tue Aug 16 10:18:43 2016

* Each table of iptables starts with a ``# Generated by ...`` line.
* Each table starts with ``*<table-name>``, for example ``*filter``.
* Each chain specifications starts with a ``:`` sign.
* A chain specification looks like ``:<chain-name> <chain-policy> [<packet-counter>:<byte-counter>]``
* The chain-name may be for example ``INPUT``.
* Each ``iptables`` rule starts with a `-` sign.

Examples:

    >>> ipt.rules[0] == {'target': 'ACCEPT', 'chain': 'INPUT', 'rule': '-m state --state RELATED,ESTABLISHED -j ACCEPT', 'table': 'filter', 'target_options': None, 'target_action': 'jump', 'constraints': '-m state --state RELATED,ESTABLISHED'}
    True
    >>> ipt.get_chain('INPUT')[1] == {'target': 'ACCEPT', 'chain': 'INPUT', 'rule': '-s 192.168.0.0/24 -j ACCEPT', 'table': 'filter', 'target_options': None, 'target_action': 'jump', 'constraints': '-s 192.168.0.0/24'}
    True
    >>> ipt.table_chains('mangle') == {'FORWARD': [], 'INPUT': [], 'POSTROUTING': [], 'PREROUTING': [], 'OUTPUT': []}
    True
    >>> ipt.get_table('nat')[-1] == {'policy': 'ACCEPT', 'table': 'nat', 'byte_counter': 450, 'name': 'OUTPUT', 'packet_counter': 3}
    True
"""

from .. import Parser, parser, get_active_lines, CommandParser
from insights.specs import Specs


class IPTablesConfiguration(Parser):
    """
    A general class for parsing iptables configuration in the
    ``iptables-save``-like format.
    """
    def parse_content(self, content):
        self.chains = []
        self.rules = []
        current_table = None
        for line in get_active_lines(content):
            if line.startswith("*"):
                current_table = line[1:].strip()
            elif line.startswith(":"):
                name, policy, counter = line[1:].split()
                packet_counter, byte_counter = counter.strip("[]").split(":")
                self.chains.append({
                    "policy": policy if policy != "-" else None,
                    "table": current_table,
                    "name": name,
                    "packet_counter": int(packet_counter),
                    "byte_counter": int(byte_counter),
                })
            elif line.startswith("-"):
                line_spl = line[3:].split(None, 1)
                if not line_spl:
                    continue
                chain_name = line_spl[0]
                rule = line_spl[1] if len(line_spl) == 2 else ''
                target_option = [i for i in (' -j', '-j ', ' -g', '-g ') if i in rule]
                if target_option:
                    constraints, target = [i.strip() for i in rule.split(target_option[-1])]
                    if " " in target:
                        target, target_options = target.split(None, 1)
                    else:
                        target_options = None
                    self.rules.append({
                        "table": current_table,
                        "chain": chain_name,
                        "rule": rule,
                        "target_action": "jump" if target_option[-1].strip() == "-j" else "goto",
                        "constraints": constraints,
                        "target": target,
                        "target_options": target_options
                    })
                else:
                    self.rules.append({
                        "table": current_table,
                        "chain": chain_name,
                        "rule": rule
                    })

    def get_chain(self, name, table="filter"):
        """
        Get the list of rules for a particular chain. Chain order is kept intact.

        Args:
            name (str): chain name, e.g. ``
            table (str): table name, defaults to ``filter``

        Returns:
            list: rules
        """
        return [r for r in self.rules if r["table"] == table and r["chain"] == name]

    def get_table(self, name="filter"):
        """
        Get the list of chains for a particular table.

        Args:
            name (str): table name, defaults to ``filter``

        Returns:
            list: chains
        """
        return [c for c in self.chains if c["table"] == name]

    def table_chains(self, table="filter"):
        """
        Get a dict where the keys are all the chains for the given table
        and each value is the set of rules defined for the given chain.

        Args:
            table (str): table name, defaults to ``filter``

        Returns:
            dict: chains with set of defined rules
        """
        return dict((c["name"], self.get_chain(c["name"], table)) for c in self.get_table(table))

    def get_rule(self, s):
        """
        Get the list of rules that contain the given string.

        Args:
            s (str): string to look for in iptables rules

        Returns:
            list: rules containing given string
        """
        return [r for r in self.rules if s in r["rule"]]

    def __contains__(self, s):
        return any(s in r["rule"] for r in self.rules)


@parser(Specs.iptables)
class IPTables(CommandParser, IPTablesConfiguration):
    """
    Process output of the ``iptables-save`` command.

    See the :py:class:`insights.parsers.iptables.IPTablesConfiguration` base
    class for additional information.
    """
    pass


@parser(Specs.ip6tables)
class IP6Tables(CommandParser, IPTablesConfiguration):
    """
    Process output of the ``ip6tables-save`` command.

    See the :py:class:`insights.parsers.iptables.IPTablesConfiguration` base
    class for additional information.
    """
    pass


@parser(Specs.iptables_permanent)
class IPTabPermanent(IPTablesConfiguration):
    """
    Process ``iptables`` configuration saved in file ``/etc/sysconfig/iptables``.

    The configuration in this file is loaded by the ``iptables`` service when the system boots.
    New configuration is saved by using the ``service iptables save`` command. This configuration
    file is not available on a system with ``firewalld`` service.

    See the :py:class:`insights.parsers.iptables.IPTablesConfiguration` base
    class for additional information.
    """
    pass


@parser(Specs.ip6tables_permanent)
class IP6TabPermanent(IPTablesConfiguration):
    """
    Process ``ip6tables`` configuration saved in file ``/etc/sysconfig/ip6tables``.

    The configuration in this file is loaded by the ``ip6tables`` service when the system boots.
    New configuration is saved by using the ``service ip6tables save`` command. This configuration
    file is not available on a system with ``firewalld`` service.

    See the :py:class:`insights.parsers.iptables.IPTablesConfiguration` base
    class for additional information.
    """
    pass



"""
mpirun Version - Command
========================

The parser for ``mpirun --version`` is included in this module.

"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.mpirun_version)
class MPIrunVersion(CommandParser):
    """
    Class for parsing the output of the ``/usr/local/bin/mpirun --version`` command.

    Sample output::

        Intel(R) MPI Library for Linux* OS, Version 2019 Update 12 Build 202010429 (id: e380127cb)
        Copyright 2003-2021, Intel Corporation.

    Examples:
        >>> mpirun_ver.year
        '2019'
        >>> mpirun_ver.version
        'Version 2019 Update 08 Build 202010429 (id: e380127cb)'
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty content.")
        if len(content) != 2:
            raise SkipComponent("Content not parsable.")

        self.year = content[0].split('Version ')[1].split(" ")[0]
        self.version = content[0].split(', ')[-1]



import json
import os
from collections import defaultdict

from mock.mock import patch

from insights import collect
from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.core import dr
from insights.core.spec_factory import encoding
from insights.core.spec_factory import RegistryPoint
from insights.core.spec_factory import safe_open
from insights.core.spec_factory import simple_file
from insights.core.spec_factory import SpecSet

tmp_file_path = "/tmp/empty_insights_unittest_with_pattern"

specs_manifest = """
---
version: 0

client:
    context:
        class: insights.core.context.HostContext
        args:
            timeout: 5

    # commands and files to ignore
    blacklist:
        files: []
        commands: []
        patterns: []
        keywords: []

    persist:
        - name: insights.tests.specs.test_specs_content_redaction_empty.Specs
          enabled: true

    run_strategy:
        name: serial
        args:
            max_workers: null

plugins:
    default_component_enabled: false

    packages:
        - insights.tests.specs.test_specs_content_redaction_empty

    configs:
        - name: insights.tests.specs.test_specs_content_redaction_empty.Specs
          enabled: true
        - name: insights.tests.specs.test_specs_content_redaction_empty.Stuff
          enabled: true
""".strip()


class Specs(SpecSet):
    file_no_redact = RegistryPoint(no_redact=True)
    file_w_redact = RegistryPoint(no_redact=False)


class Stuff(Specs):
    file_no_redact = simple_file(tmp_file_path, save_as=tmp_file_path + '_1')
    file_w_redact = simple_file(tmp_file_path)


#
# TEST
#


def setup_function(func):
    with safe_open(tmp_file_path, "w+", encoding=encoding, errors="surrogateescape") as f:
        f.write('\n'.join(["KEEEY {0}".format(i) for i in range(3)]))


def teardown_function(func):
    if os.path.exists(tmp_file_path):
        os.remove(tmp_file_path)
    # Reset Test ENV
    dr.COMPONENTS = defaultdict(lambda: defaultdict(set))
    dr.TYPE_OBSERVERS = defaultdict(set)
    dr.ENABLED = defaultdict(lambda: True)


@patch('insights.cleaner.Cleaner.generate_report')
def test_specs_ds_with_hn_collect(mock_fun):
    # Preparation
    manifest = collect.load_manifest(specs_manifest)
    for pkg in manifest.get("plugins", {}).get("packages", []):
        dr.load_components(pkg, exclude=None)
    # For verifying convenience, test obfuscate=False only
    conf = InsightsConfig(manifest=manifest)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()
    rm_conf = {'patterns': {'regex': ['KEEEY']}, 'keywords': ['TeST']}
    output_path, errors = collect.collect(
        tmp_path=arch.tmp_dir, archive_name=arch.archive_name, rm_conf=rm_conf, client_config=conf
    )
    meta_data_root = os.path.join(output_path, 'meta_data')
    data_root = os.path.join(output_path, 'data')

    assert not errors
    spec_count = 0
    line_count = 0
    for spec in Specs.__dict__:
        if not spec.startswith(('__', 'context_handlers', 'registry')):
            file_name = (
                "insights.tests.specs.test_specs_content_redaction_empty.Specs.{0}.json".format(
                    spec
                )
            )
            meta_data = os.path.join(meta_data_root, file_name)
            with open(meta_data, 'r') as fp:
                mdata = json.load(fp)
                results = mdata.get('results')
                if 'file_no_redact' in spec:
                    spec_count += 1
                    assert not mdata.get('errors')
                    assert results
                    if not isinstance(results, list):
                        results = [results]
                    for result in results:
                        rel = result['object']['relative_path']
                        with open(os.path.join(data_root, rel), 'r') as fp:
                            for line in fp:
                                line_count += 1
                                assert "KEEEY" in line
                else:
                    spec_count += 1
                    # file_w_redact is not collected as it's redacted to empty
                    assert mdata.get('errors')
                    assert not results
                    assert not os.path.exists(os.path.join(data_root, tmp_file_path.lstrip('/')))

    assert spec_count == 2  # Number of Specs
    assert line_count == 3  # Number of Lines in "file_no_redact"

    arch.delete_archive_dir()



# -*- coding: utf-8 -*-
import pytest
from insights.util.rpm_vercmp import _rpm_vercmp, version_compare


# data copied from
# https://raw.githubusercontent.com/rpm-software-management/rpm/master/tests/rpmvercmp.at

DEFAULT_DATA = """
m4_define([RPMVERCMP],[
AT_SETUP([rpmvercmp($1, $2) = $3])
AT_KEYWORDS([vercmp])
AT_CHECK([
AT_SKIP_IF([$LUA_DISABLED])
runroot rpm --eval '%{lua: print(rpm.vercmp("$1", "$2"))}'], [0], [$3
], [])
AT_CLEANUP
])

AT_BANNER([RPM version comparison])

RPMVERCMP(1.0, 1.0, 0)
RPMVERCMP(1.0, 2.0, -1)
RPMVERCMP(2.0, 1.0, 1)

RPMVERCMP(2.0.1, 2.0.1, 0)
RPMVERCMP(2.0, 2.0.1, -1)
RPMVERCMP(2.0.1, 2.0, 1)

RPMVERCMP(2.0.1a, 2.0.1a, 0)
RPMVERCMP(2.0.1a, 2.0.1, 1)
RPMVERCMP(2.0.1, 2.0.1a, -1)

RPMVERCMP(5.5p1, 5.5p1, 0)
RPMVERCMP(5.5p1, 5.5p2, -1)
RPMVERCMP(5.5p2, 5.5p1, 1)

RPMVERCMP(5.5p10, 5.5p10, 0)
RPMVERCMP(5.5p1, 5.5p10, -1)
RPMVERCMP(5.5p10, 5.5p1, 1)

RPMVERCMP(10xyz, 10.1xyz, -1)
RPMVERCMP(10.1xyz, 10xyz, 1)

RPMVERCMP(xyz10, xyz10, 0)
RPMVERCMP(xyz10, xyz10.1, -1)
RPMVERCMP(xyz10.1, xyz10, 1)

RPMVERCMP(xyz.4, xyz.4, 0)
RPMVERCMP(xyz.4, 8, -1)
RPMVERCMP(8, xyz.4, 1)
RPMVERCMP(xyz.4, 2, -1)
RPMVERCMP(2, xyz.4, 1)

RPMVERCMP(5.5p2, 5.6p1, -1)
RPMVERCMP(5.6p1, 5.5p2, 1)

RPMVERCMP(5.6p1, 6.5p1, -1)
RPMVERCMP(6.5p1, 5.6p1, 1)

RPMVERCMP(6.0.rc1, 6.0, 1)
RPMVERCMP(6.0, 6.0.rc1, -1)

RPMVERCMP(10b2, 10a1, 1)
RPMVERCMP(10a2, 10b2, -1)

RPMVERCMP(1.0aa, 1.0aa, 0)
RPMVERCMP(1.0a, 1.0aa, -1)
RPMVERCMP(1.0aa, 1.0a, 1)

RPMVERCMP(10.0001, 10.0001, 0)
RPMVERCMP(10.0001, 10.1, 0)
RPMVERCMP(10.1, 10.0001, 0)
RPMVERCMP(10.0001, 10.0039, -1)
RPMVERCMP(10.0039, 10.0001, 1)

RPMVERCMP(4.999.9, 5.0, -1)
RPMVERCMP(5.0, 4.999.9, 1)

RPMVERCMP(20101121, 20101121, 0)
RPMVERCMP(20101121, 20101122, -1)
RPMVERCMP(20101122, 20101121, 1)

RPMVERCMP(2_0, 2_0, 0)
RPMVERCMP(2.0, 2_0, 0)
RPMVERCMP(2_0, 2.0, 0)

dnl RhBug:178798 case
RPMVERCMP(a, a, 0)
RPMVERCMP(a+, a+, 0)
RPMVERCMP(a+, a_, 0)
RPMVERCMP(a_, a+, 0)
RPMVERCMP(+a, +a, 0)
RPMVERCMP(+a, _a, 0)
RPMVERCMP(_a, +a, 0)
RPMVERCMP(+_, +_, 0)
RPMVERCMP(_+, +_, 0)
RPMVERCMP(_+, _+, 0)
RPMVERCMP(+, _, 0)
RPMVERCMP(_, +, 0)

dnl Basic testcases for tilde sorting
RPMVERCMP(1.0~rc1, 1.0~rc1, 0)
RPMVERCMP(1.0~rc1, 1.0, -1)
RPMVERCMP(1.0, 1.0~rc1, 1)
RPMVERCMP(1.0~rc1, 1.0~rc2, -1)
RPMVERCMP(1.0~rc2, 1.0~rc1, 1)
RPMVERCMP(1.0~rc1~git123, 1.0~rc1~git123, 0)
RPMVERCMP(1.0~rc1~git123, 1.0~rc1, -1)
RPMVERCMP(1.0~rc1, 1.0~rc1~git123, 1)

dnl Basic testcases for caret sorting
RPMVERCMP(1.0^, 1.0^, 0)
RPMVERCMP(1.0^, 1.0, 1)
RPMVERCMP(1.0, 1.0^, -1)
RPMVERCMP(1.0^git1, 1.0^git1, 0)
RPMVERCMP(1.0^git1, 1.0, 1)
RPMVERCMP(1.0, 1.0^git1, -1)
RPMVERCMP(1.0^git1, 1.0^git2, -1)
RPMVERCMP(1.0^git2, 1.0^git1, 1)
RPMVERCMP(1.0^git1, 1.01, -1)
RPMVERCMP(1.01, 1.0^git1, 1)
RPMVERCMP(1.0^20160101, 1.0^20160101, 0)
RPMVERCMP(1.0^20160101, 1.0.1, -1)
RPMVERCMP(1.0.1, 1.0^20160101, 1)
RPMVERCMP(1.0^20160101^git1, 1.0^20160101^git1, 0)
RPMVERCMP(1.0^20160102, 1.0^20160101^git1, 1)
RPMVERCMP(1.0^20160101^git1, 1.0^20160102, -1)

dnl Basic testcases for tilde and caret sorting
RPMVERCMP(1.0~rc1^git1, 1.0~rc1^git1, 0)
RPMVERCMP(1.0~rc1^git1, 1.0~rc1, 1)
RPMVERCMP(1.0~rc1, 1.0~rc1^git1, -1)
RPMVERCMP(1.0^git1~pre, 1.0^git1~pre, 0)
RPMVERCMP(1.0^git1, 1.0^git1~pre, 1)
RPMVERCMP(1.0^git1~pre, 1.0^git1, -1)

dnl These are included here to document current, arguably buggy behaviors
dnl for reference purposes and for easy checking against  unintended
dnl behavior changes.
dnl
dnl AT_BANNER([RPM version comparison oddities])
dnl RhBug:811992 case
dnl RPMVERCMP(1b.fc17, 1b.fc17, 0)
dnl RPMVERCMP(1b.fc17, 1.fc17, -1)
dnl RPMVERCMP(1.fc17, 1b.fc17, 1)
dnl RPMVERCMP(1g.fc17, 1g.fc17, 0)
dnl RPMVERCMP(1g.fc17, 1.fc17, 1)
dnl RPMVERCMP(1.fc17, 1g.fc17, -1)

dnl Non-ascii characters are considered equal so these are all the same, eh...
dnl RPMVERCMP(1.1.α, 1.1.α, 0)
dnl RPMVERCMP(1.1.α, 1.1.β, 0)
dnl RPMVERCMP(1.1.β, 1.1.α, 0)
dnl RPMVERCMP(1.1.αα, 1.1.α, 0)
dnl RPMVERCMP(1.1.α, 1.1.ββ, 0)
dnl RPMVERCMP(1.1.ββ, 1.1.αα, 0)
"""


def convert(data):
    f = "RPMVERCMP("
    lines = [l for l in data.splitlines() if f in l]
    tuples = [
        tuple(c.strip() for c in l[len(f) + l.find(f) :].rstrip(")").split(",")) for l in lines
    ]
    return [(l, r, int(i)) for (l, r, i) in tuples]


@pytest.fixture
def rpm_data():
    return convert(DEFAULT_DATA)


def test_rpm_vercmp(rpm_data):
    for l, r, expected in rpm_data:
        actual = _rpm_vercmp(l, r)
        assert actual == expected, (l, r, actual, expected)


def test_version_compare():
    rpm1 = 'kernel-rt-debug-3.10.0-327.rt56.204.el7_2.1'
    rpm2 = 'kernel-rt-debug-3.10.0-327.rt56.204.el7_2.2'
    rpm3 = 'kernel-3.10.0-327.10.1.el7'
    rpm4 = 'kernel-3.10.0-327.el7'
    rpm5 = 'kernel-3.10.1-327.el7'
    assert version_compare(rpm1, rpm2) == -1
    assert version_compare(rpm3, rpm4) == 1
    assert version_compare(rpm4, rpm5) == -1



"""
SwiftLog - file ``/var/log/containers/swift/swift.log`` and ``/var/log/swift/swift.log``
========================================================================================
"""

from insights import parser, Syslog
from insights.specs import Specs


@parser(Specs.swift_log)
class SwiftLog(Syslog):
    """Class for parsing ``/var/log/containers/swift/swift.log`` and
    ``/var/log/swift/swift.log`` file.

    Provide access to swift log using the
    :class:`insights.core.Syslog` parser class.

    Sample ``swift.log`` file content::

      Sep 29 23:50:29 rh-server object-server: Starting object replication pass.
      Sep 29 23:50:29 rh-server object-server: Nothing replicated for 0.01691198349 seconds.
      Sep 29 23:50:29 rh-server object-server: Object replication complete. (0.00 minutes)
      Sep 29 23:50:38 rh-server container-server: Beginning replication run
      Sep 29 23:50:38 rh-server container-server: Replication run OVER
      Sep 29 23:50:38 rh-server container-server: Attempted to replicate 0 dbs in 0.00064 seconds (0.00000/s)

    Examples:

      >>> obj_server_lines = swift_log.get("object-server")
      >>> len(obj_server_lines)
      3
      >>> obj_server_lines[0].get("procname")
      'object-server'
      >>> obj_server_lines[0].get("message")
      'Starting object replication pass.'

    """
    pass



"""
DnfConf - file ``/etc/dnf/dnf.conf``
====================================

This module provides parsing for the ``/etc/dnf/dnf.conf`` file.
The ``DnfConf`` class parses the information in the file
``/etc/dnf/dnf.conf``. See the ``YumConf`` class for more
information on attributes and methods.

Sample file content::

    [main]
    gpgcheck=1
    installonly_limit=3
    clean_requirements_on_remove=True
    best=False
    skip_if_unavailable=True

Examples:
    >>> 'main' in dconf
    True
    >>> 'rhel-7-server-rpms' in dconf
    False
    >>> dconf.has_option('main', 'gpgcheck')
    True
    >>> dconf.has_option('main', 'foo')
    False
"""

from insights import parser
from insights.specs import Specs
from insights.core.filters import add_filter
from insights.parsers.yum_conf import YumConf

add_filter(Specs.dnf_conf, '[')


@parser(Specs.dnf_conf)
class DnfConf(YumConf):
    """Parse contents of file ``/etc/dnf/dnf.conf``."""
    pass



from insights.client.connection import InsightsConnection
from mock.mock import Mock, patch
from os import environ as os_environ


def test_get_proxy_env():
    """
    Test geting proxy configuration through environment, with and without no_proxy setting.
    """
    config = Mock()
    config.no_proxy = None
    config.proxy = None
    config.base_url = "redhat.com"

    connection = InsightsConnection(config)

    with patch.dict(os_environ, {"HTTPS_PROXY": "env.proxy.example.com"}, clear=True):
        connection.get_proxies()
    assert connection.proxies == {'https': 'env.proxy.example.com'}


def test_get_proxy_rhsm():
    """
    Test getting proxy configuration through rhsm (auto-config), with and without no_proxy setting.
    """
    config = Mock()
    config.no_proxy = None
    config.base_url = "redhat.com"
    config.proxy = "rhsm.proxy.example.com"

    connection = InsightsConnection(config)

    connection.get_proxies()
    assert connection.proxies == {"https": config.proxy}


def test_get_no_proxy_env():
    """
    Test geting proxy configuration through environment with no_proxy modification.
    """

    config = Mock()
    config.no_proxy = None
    config.proxy = None
    config.base_url = "redhat.com"

    connection = InsightsConnection(config)

    with patch.dict(os_environ, {"HTTPS_PROXY": "env.proxy.example.com", "NO_PROXY": "*"}, clear=True):
        connection.get_proxies()
    assert connection.proxies is None

    with patch.dict(os_environ, {"HTTPS_PROXY": "env.proxy.example.com", "NO_PROXY": "redhat.com"}, clear=True):
        connection.get_proxies()
    assert connection.proxies is None

    with patch.dict(os_environ, {"HTTPS_PROXY": "env.proxy.example.com", "NO_PROXY": "redhat.com,example.com"}, clear=True):
        connection.get_proxies()
    assert connection.proxies is None

    with patch.dict(os_environ, {"HTTPS_PROXY": "env.proxy.example.com", "NO_PROXY": "url.com,example.com"}, clear=True):
        connection.get_proxies()
    assert connection.proxies == {'https': 'env.proxy.example.com'}

    with patch.dict(os_environ, {"HTTPS_PROXY": "env.proxy.example.com", "NO_PROXY": "url.com"}, clear=True):
        connection.get_proxies()
    assert connection.proxies == {'https': 'env.proxy.example.com'}


def test_get_no_proxy_rhsm():
    """
    Test geting proxy configuration through rhsm (auto-config) with no_proxy modification.
    """
    config = Mock()
    config.proxy = "rhsm.proxy.example.com"
    config.base_url = "redhat.com"
    config.no_proxy = ""

    connection = InsightsConnection(config)

    config.no_proxy = "*"
    connection.get_proxies()
    assert connection.proxies is None

    config.no_proxy = "redhat.com"
    connection.get_proxies()
    assert connection.proxies is None

    config.no_proxy = "url.com"
    connection.get_proxies()
    assert connection.proxies == {"https": config.proxy}

    config.no_proxy = "example.com,url.com"
    connection.get_proxies()
    assert connection.proxies == {"https": config.proxy}

    config.no_proxy = "example.com, url.com"
    connection.get_proxies()
    assert connection.proxies == {"https": config.proxy}

    config.no_proxy = "example.com , url.com"
    connection.get_proxies()
    assert connection.proxies == {"https": config.proxy}

    config.no_proxy = "redhat.com, example.com"
    connection.get_proxies()
    assert connection.proxies is None

    config.no_proxy = "example.com, redhat.com"
    connection.get_proxies()
    assert connection.proxies is None

    config.no_proxy = "redhat.com,example.com"
    connection.get_proxies()
    assert connection.proxies is None

    config.no_proxy = "example.com,redhat.com"
    connection.get_proxies()
    assert connection.proxies is None

    config.no_proxy = "example.com , redhat.com"
    connection.get_proxies()
    assert connection.proxies is None


@patch("insights.client.connection.logger.debug")
def test_get_rhsm_and_env(logger):
    """
    no_proxy from environment doesn't change the proxy configuration from auto-config
    """
    config = Mock()
    config.proxy = "rhsm.proxy.example.com"
    config.no_proxy = None
    config.base_url = "redhat.com"

    connection = InsightsConnection(config)

    with patch.dict(os_environ, {"NO_PROXY": "redhat.com"}, clear=True):
        connection.get_proxies()
    assert connection.proxies == {"https": config.proxy}



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import sys_class
from insights.parsers.sys_class import TtyConsoleActive
from insights.tests import context_wrap


TTY_CONSOLE_ACTIVE = """
tty0 ttyS0
""".strip()

TTY_CONSOLE_ACTIVE_EMPTY = ""

TTY_CONSOLE_ACTIVE_MULTI_LINES = """
tty0 ttyS0
tty0 ttyS0
""".strip()


def test_doc():
    env = {
        'tty_console_active': TtyConsoleActive(context_wrap(TTY_CONSOLE_ACTIVE))
    }
    failed, total = doctest.testmod(sys_class, globs=env)
    assert failed == 0


def test_tty_console_active():
    tty_console_active = TtyConsoleActive(context_wrap(TTY_CONSOLE_ACTIVE))
    assert tty_console_active.devices == ['tty0', 'ttyS0']


def test_class_exceptions():
    with pytest.raises(SkipComponent) as e:
        TtyConsoleActive(context_wrap(TTY_CONSOLE_ACTIVE_MULTI_LINES))
    assert 'This should be an one line file' in str(e)

    with pytest.raises(SkipComponent) as e:
        TtyConsoleActive(context_wrap(TTY_CONSOLE_ACTIVE_EMPTY))
    assert 'Empty content' in str(e)



import pytest
from insights.core.exceptions import SkipComponent
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.cmdline import CmdLine
from insights.parsers.systemd.unitfiles import ListUnits
from insights.parsers.redhat_release import RedhatRelease
from insights.parsers.rpm_ostree_status import RpmOstreeStatus
from insights.combiners.rhel_for_edge import RhelForEdge
from insights.combiners import rhel_for_edge
from insights.tests import context_wrap
import doctest

CONTENT_REDHAT_RELEASE_RHEL = """
Red Hat Enterprise Linux release 8.4 (Ootpa)
""".strip()

CONTENT_REDHAT_RELEASE_COREOS = """
Red Hat Enterprise Linux CoreOS release 4.12
""".strip()

CMDLINE_RHEL = """
BOOT_IMAGE=/vmlinuz-4.18.0-80.el8.x86_64 root=/dev/mapper/rootvg-rootlv ro crashkernel=184M rd.lvm.lv=rootvg/rootlv biosdevname=0 printk.time=1 numa=off audit=1 transparent_hugepage=always LANG=en_US.UTF-8
""".strip()

CMDLINE_EDGE = """
BOOT_IMAGE=(hd0,msdos1)/ostree/rhel-323453435435234234232534534/vmlinuz-4.18.0-348.20.1.el8_5.x86_64 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap ostree=/ostree/boot.1/rhel/08987978979/0 rhgb quiet LANG=en_US.UTF-8
""".strip()

CONTENT_SYSTEMCTL_LIST_UNITS_NO_AUTOMATED = """
postgresql.service                                                                               loaded active running   PostgreSQL database server
""".strip()

CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED = """
postgresql.service                                                                               loaded active running   PostgreSQL database server
rhcd.service                                                                                     loaded active running   Red Hat connector daemon
""".strip()

CONTENT_INSTALLED_RPMS_RHEL = """
kernel-4.18.0-305.19.1.el8_4.x86_64
""".strip()

CONTENT_INSTALLED_RPMS_EDGE = """
kernel-4.18.0-305.19.1.el8_4.x86_64
rpm-ostree-2021.5-2.el8.x86_64
""".strip()

RPM_OSTREE_STATUS_EDGE = """
{
  "deployments" : [
    {
      "base-commit-meta" : {
        "rpmostree.inputhash" : "156ae2e3ad99777a0b7e48d169ee2f4fa618fd76a4ffc5ba8af8fad8c46ea75d"
      },
      "unlocked" : "none",
      "booted" : true,
      "initramfs-etc" : [
      ],
      "id" : "rhel-7c25253d45a30b05222fba1365fce35263987262c490ad547e66d8541f05293a.0",
      "osname": "rhel",
      "origin" : "edge:rhel/8/x86_64/edge",
    },
    {
      "base-commit-meta" : {
        "rpmostree.inputhash" : "f2ff223ab7b3cc45ca20fadc0a138808cc084a20bc5ce9f22b451a802c652c26",
        "ostree.linux" : "4.18.0-372.9.1.el8.x86_64",
        "ostree.bootable" : true
      },
      "id" : "rhel-e2707b5fc7d4b168d7746678bc228b734dbf6f9f64b8ad97f5c87f7b9e7dfea7.0",
      "osname": "rhel",
      "pending-base-checksum" : "a35ac0ae55eb10f2f29e3f01cc760ae3ad5341d44a285129704749f24233d004",
      "origin" : "edge:rhel/8/x86_64/edge",
      "regenerate-initramfs" : false,
      "checksum" : "e2707b5fc7d4b168d7746678bc228b734dbf6f9f64b8ad97f5c87f7b9e7dfea7",
    }
  ],
  "transaction" : null,
  "cached-update" : null
}
""".strip()

RPM_OSTREE_STATUS_RHCOS = """
{
  "deployments" : [
    {
      "requested-local-packages" : [
      ],
      "base-commit-meta" : {
        "ostree.manifest-digest" : "sha256:90780b08939d913d9cfc5565eecdd12055dd57301b2109985e149830a92ed6b1",
        "ostree.importer.version" : "0.8.7",
        "ostree.tar-filtered" : {
        }
      },
      "base-removals" : [
      ],
      "unlocked" : "none",
      "booted" : true,
      "requested-local-fileoverride-packages" : [
      ],
      "requested-modules-enabled" : [
      ],
      "id" : "rhcos-40948b845a13349cb0d57400eebdcfdf72bb8521636a1f067fb22d92b0b73b41.0",
      "base-remote-replacements" : {
      },
      "osname" : "rhcos",
      "pinned" : false,
      "regenerate-initramfs" : false,
      "base-local-replacements" : [
      ],
      "container-image-reference" : "ostree-unverified-registry:quay.io/openshift-release-dev/ocp-v4.0-art-dev@sha256:90780b08939d913d9cfc5565eecdd12055dd57301b2109985e149830a92ed6b1",
      "checksum" : "40948b845a13349cb0d57400eebdcfdf72bb8521636a1f067fb22d92b0b73b41",
      "requested-base-local-replacements" : [
      ],
      "requested-modules" : [
      ],
      "requested-packages" : [
      ],
      "serial" : 0,
      "timestamp" : 1674005544,
      "packages" : [
      ],
      "staged" : false,
      "requested-base-removals" : [
      ],
      "modules" : [
      ],
      "container-image-reference-digest" : "sha256:90780b08939d913d9cfc5565eecdd12055dd57301b2109985e149830a92ed6b1"
    }
  ],
  "transaction" : null,
  "cached-update" : null,
  "update-driver" : null
}
""".strip()

RPM_OSTREE_STATUS_EDGE_FALSE = """
{
  "deployments" : [
    {
      "base-commit-meta" : {
        "rpmostree.inputhash" : "156ae2e3ad99777a0b7e48d169ee2f4fa618fd76a4ffc5ba8af8fad8c46ea75d"
      },
      "requested-local-packages" : [
      ],
      "base-removals" : [
      ],
      "unlocked" : "none",
      "booted" : true,
      "initramfs-etc" : [
      ],
      "id" : "rhel-7c25253d45a30b05222fba1365fce35263987262c490ad547e66d8541f05293a.0",
      "layered-commit-meta" : {
        "rpmostree.packages" : [
          "glibc-langpack-en",
          "ksh",
          "wget",
          "rhc"
        ],
        "rpmostree.state-sha512" : "c81ae53960de1009a1bace2d89fe58d42790147ba8018e1799ea22bf6f74e754af57530e04d092ed2975bcfaf7861d20c7133268ad02b8644973710727a1d311",
        "rpmostree.removed-base-packages" : [
        ],
        "ostree.linux" : "4.18.0-348.el8.x86_64",
        "rpmostree.rpmmd-repos" : [
          {
            "id" : "rhel-8-for-x86_64-baseos-rpms",
            "timestamp" : 1665041964
          },
          {
            "id" : "rhel-8-for-x86_64-appstream-rpms",
            "timestamp" : 1665042074
          }
        ],
        "rpmostree.modules" : [
        ],
        "rpmostree.clientlayer_version" : 5,
        "ostree.bootable" : true,
        "rpmostree.clientlayer" : true,
        "rpmostree.replaced-base-packages" : [
        ]
      },
      "osname": "rhel",
      "origin" : "edge:rhel/8/x86_64/falsetest",
      "pinned" : false,
      "regenerate-initramfs" : false,
      "base-timestamp" : 1665394938,
      "base-local-replacements" : [
      ],
      "checksum" : "7c25253d45a30b05222fba1365fce35263987262c490ad547e66d8541f05293a",
      "requested-base-local-replacements" : [
      ],
      "timestamp" : 1665398315,
      "requested-packages" : [
        "glibc-langpack-en",
        "ksh",
        "wget",
        "rhc"
      ],
      "serial" : 0,
      "packages" : [
        "glibc-langpack-en",
        "ksh",
        "wget",
        "rhc"
      ],
      "base-checksum" : "a35ac0ae55eb10f2f29e3f01cc760ae3ad5341d44a285129704749f24233d004",
      "gpg-enabled" : false,
      "requested-base-removals" : [
      ]
    },
    {
      "base-commit-meta" : {
        "rpmostree.inputhash" : "f2ff223ab7b3cc45ca20fadc0a138808cc084a20bc5ce9f22b451a802c652c26",
        "ostree.linux" : "4.18.0-372.9.1.el8.x86_64",
        "ostree.bootable" : true
      },
      "requested-local-packages" : [
      ],
      "base-removals" : [
      ],
      "unlocked" : "none",
      "booted" : false,
      "initramfs-etc" : [
      ],
      "id" : "rhel-e2707b5fc7d4b168d7746678bc228b734dbf6f9f64b8ad97f5c87f7b9e7dfea7.0",
      "layered-commit-meta" : {
        "rpmostree.packages" : [
          "glibc-langpack-en",
          "ksh",
          "wget",
          "rhc"
        ],
        "rpmostree.state-sha512" : "a750200ccd47616f629c5c3c3937b51f561c1f1f7fee7dd94295997e0bf16e756b0eb8517f37e1f78f06bf281c6d831745e82765ffe23dd3248038511ae96df1",
        "rpmostree.removed-base-packages" : [
        ],
        "ostree.linux" : "4.18.0-372.9.1.el8.x86_64",
        "rpmostree.rpmmd-repos" : [
          {
            "id" : "rhel-8-for-x86_64-appstream-rpms",
            "timestamp" : 1663855048
          },
          {
            "id" : "rhel-8-for-x86_64-baseos-rpms",
            "timestamp" : 1663584426
          }
        ],
        "rpmostree.modules" : [
        ],
        "rpmostree.clientlayer_version" : 5,
        "ostree.bootable" : true,
        "rpmostree.clientlayer" : true,
        "rpmostree.replaced-base-packages" : [
        ]
      },
      "osname": "rhel",
      "pending-base-checksum" : "a35ac0ae55eb10f2f29e3f01cc760ae3ad5341d44a285129704749f24233d004",
      "origin" : "edge:rhel/8/x86_64/falsetest",
      "regenerate-initramfs" : false,
      "pending-base-timestamp" : 1665394938,
      "base-timestamp" : 1652270522,
      "base-local-replacements" : [
      ],
      "checksum" : "e2707b5fc7d4b168d7746678bc228b734dbf6f9f64b8ad97f5c87f7b9e7dfea7",
      "requested-base-local-replacements" : [
      ],
      "timestamp" : 1663923873,
      "requested-packages" : [
        "glibc-langpack-en",
        "ksh",
        "wget",
        "rhc"
      ],
      "serial" : 0,
      "packages" : [
        "glibc-langpack-en",
        "ksh",
        "wget",
        "rhc"
      ],
      "pinned" : false,
      "base-checksum" : "d82856c783ddb1c6ad9f728e60bec2b39626eef12984860c4a92fea04b1d396e",
      "gpg-enabled" : false,
      "requested-base-removals" : [
      ]
    }
  ],
  "transaction" : null,
  "cached-update" : null
}
""".strip()


def test_rhel_for_edge_true_1():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_NO_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))
    rpm_ostree_status = RpmOstreeStatus(context_wrap(RPM_OSTREE_STATUS_EDGE))

    result = RhelForEdge(list_units, rpm_ostree_status, install_rpms, cmdline, redhat_release)
    assert result.is_edge is True
    assert result.is_automated is False


def test_rhel_for_edge_true_2():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    result = RhelForEdge(list_units, None, install_rpms, cmdline, redhat_release)
    assert result.is_edge is True
    assert result.is_automated is True


def test_rhel_for_edge_false_1():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_RHEL))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    result = RhelForEdge(list_units, None, install_rpms, cmdline, redhat_release)
    assert result.is_edge is False
    assert result.is_automated is False


def test_rhel_for_edge_false_2():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_RHEL))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    result = RhelForEdge(list_units, None, install_rpms, cmdline, redhat_release)
    assert result.is_edge is False
    assert result.is_automated is False


def test_rhel_for_edge_false_3():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_COREOS))

    result = RhelForEdge(list_units, None, install_rpms, cmdline, redhat_release)
    assert result.is_edge is False
    assert result.is_automated is False


def test_rhel_for_edge_false_4():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_COREOS))
    rpm_ostree_status = RpmOstreeStatus(context_wrap(RPM_OSTREE_STATUS_RHCOS))

    result = RhelForEdge(list_units, rpm_ostree_status, install_rpms, cmdline, redhat_release)
    assert result.is_edge is False
    assert result.is_automated is False


def test_rhel_for_edge_false_5():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_COREOS))
    rpm_ostree_status = RpmOstreeStatus(context_wrap(RPM_OSTREE_STATUS_EDGE_FALSE))

    result = RhelForEdge(list_units, rpm_ostree_status, install_rpms, cmdline, redhat_release)
    assert result.is_edge is False
    assert result.is_automated is False


def test_rhel_for_edge_false_6():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))

    with pytest.raises(SkipComponent) as e:
        RhelForEdge(list_units, None, install_rpms, cmdline, None)
    assert "Unable to determine if this system is created from an edge image" in str(e)


def test_doc_examples():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    env = {
            'rhel_for_edge_obj': RhelForEdge(list_units, None, install_rpms, cmdline, redhat_release)
          }
    failed, total = doctest.testmod(rhel_for_edge, globs=env)
    assert failed == 0



from insights.tests import context_wrap
from insights.parsers.sysconfig import SbdSysconfig

SBD_SYSCONFIG = """
SBD_DEVICE="/dev/disk/by-path/ip-198.20.25.163:3260-iscsi-iqn.2022-09.com.epmpttest:rhel85-lun-0"
SBD_DELAY_START=no
SBD_OPTS="-n emptytest"
SBD_PACEMAKER=yes
SBD_STARTMODE=always
SBD_WATCHDOG_DEV=/dev/watchdog
SBD_WATCHDOG_TIMEOUT=5
""".strip()


def test_sysconfig_sbd():
    result = SbdSysconfig(context_wrap(SBD_SYSCONFIG))
    assert result["SBD_DELAY_START"] == 'no'
    assert result.get("SBD_PACEMAKER") == 'yes'
    assert 'SBD_WATCHDOG_TIMEOUT' in result
    assert result.get('SBD_WATCHDOG_TIMEOUT') == '5'
    assert "OPTIONS1" not in result



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import numa_cpus
from insights.parsers.numa_cpus import NUMACpus
from insights.tests import context_wrap

NODE0_PATH = "/sys/devices/system/node/node0/cpulist"
NODE1_PATH = "/sys/devices/system/node/node1/cpulist"
NODE0_PATH_NO = ""

NODE0_CPULIST_RANGE = """
0-6,14-20
""".strip()

NODE1_CPULIST_RANGE = """
""".strip()

NODE0_CPULIST_RANGE_1 = """
0-3
""".strip()
NODE1_CPULIST_RANGE_2 = """
4-7
""".strip()

NODE0_CPULIST = """
0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,32,34,36,38,40,42,44,46
""".strip()

NODE1_CPULIST = """
1,3,5,7,9,11,13,15,17,19,21,23,25,27,29,31,33,35,37,39,41,43,45,47
""".strip()


def test_cpulist_node0():
    context = context_wrap(NODE1_CPULIST, NODE1_PATH)
    cpu_obj = NUMACpus(context)
    assert cpu_obj.numa_node_details() == {'numa_node_range': ['1', '3', '5', '7', '9', '11', '13', '15', '17', '19', '21', '23', '25', '27', '29', '31', '33', '35', '37', '39', '41', '43', '45', '47'], 'numa_node_name': 'node1', 'total_cpus': 24}
    assert cpu_obj.numa_node_name == 'node1'
    assert len(cpu_obj.numa_node_cpus) == 24
    assert cpu_obj.total_numa_node_cpus == 24

    context = context_wrap(NODE0_CPULIST_RANGE, NODE0_PATH)
    cpu_obj = NUMACpus(context)
    assert cpu_obj.numa_node_details() == {'numa_node_range': ['0-6', '14-20'], 'numa_node_name': 'node0', 'total_cpus': 14}
    assert cpu_obj.numa_node_name == 'node0'
    assert cpu_obj.numa_node_cpus == ['0-6', '14-20']
    assert cpu_obj.total_numa_node_cpus == 14

    context = context_wrap(NODE1_CPULIST_RANGE_2, NODE1_PATH)
    cpu_obj = NUMACpus(context)
    assert cpu_obj.numa_node_details() == {'numa_node_range': ['4-7'], 'numa_node_name': 'node1', 'total_cpus': 4}
    assert cpu_obj.numa_node_name == 'node1'
    assert cpu_obj.numa_node_cpus == ['4-7']
    assert cpu_obj.total_numa_node_cpus == 4

    with pytest.raises(SkipComponent) as exc:
        cpu_obj = NUMACpus(context_wrap(NODE1_CPULIST_RANGE, NODE1_PATH))
        assert cpu_obj.numa_node_name == 'node1'
        assert not cpu_obj.total_numa_node_cpus
    assert 'No Contents' in str(exc)


def test_numa_node_doc_examples():
    env = {'numa_cpus_obj': NUMACpus(context_wrap(NODE0_CPULIST_RANGE, NODE0_PATH))}
    failed, total = doctest.testmod(numa_cpus, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import sap_hdb_version
from insights.parsers.sap_hdb_version import HDBVersion
from insights.tests import context_wrap

HDB_VER_1 = """
HDB version info:
  version:             2.00.030.00.1522210459
  branch:              hanaws
  machine config:      linuxx86_64
  git hash:            bb2ff6b25b8eab5ab382c170a43dc95ae6ce298f
  git merge time:      2018-03-28 06:14:19
  weekstone:           2018.13.0
  cloud edition:       0000.00.00
  compile date:        2018-03-28 06:19:13
  compile host:        ld2221
  compile type:        rel
""".strip()

HDB_VER_2 = """
HDB version info:
  version:             2.00.020.00.1500920972
  branch:              fa/hana2sp02
  git hash:            7f63b0aa11dca2ea54d450aa302319302c2eeaca
  git merge time:      2017-07-24 20:29:32
  weekstone:           0000.00.0
  compile date:        2017-07-24 20:35:12
  compile host:        ld4551
  compile type:        rel
""".strip()

HDB_VER_NG_1 = """
su: user hxeadm does not exist
""".strip()

HDB_VER_NG_2 = """
HDB version info:
  version:             2.00.020.1500920972
  branch:              fa/hana2sp02
  git hash:            7f63b0aa11dca2ea54d450aa302319302c2eeaca
  git merge time:      2017-07-24 20:29:32
  weekstone:           0000.00.0
  compile date:        2017-07-24 20:35:12
  compile host:        ld4551
""".strip()

CMD_PATH = "insights_commands/sudo_-iu_sr1adm_HDB_version"


def test_HDBVersion_doc():
    env = {
            'hdb_ver': HDBVersion(context_wrap(HDB_VER_1, path=CMD_PATH)),
          }
    failed, total = doctest.testmod(sap_hdb_version, globs=env)
    assert failed == 0


def test_HDBVersion_ng():
    with pytest.raises(SkipComponent) as e_info:
        HDBVersion(context_wrap(HDB_VER_NG_1))
    assert "Incorrect content." in str(e_info.value)

    with pytest.raises(SkipComponent) as e_info:
        HDBVersion(context_wrap(HDB_VER_NG_2))
    assert "Incorrect HDB version: 2.00.020.1500920972" in str(e_info.value)


def test_HDBVersion_1():
    hdb_ver = HDBVersion(context_wrap(HDB_VER_1, path=CMD_PATH))
    assert hdb_ver['branch'] == 'hanaws'
    assert hdb_ver['compile type'] == 'rel'
    assert hdb_ver['weekstone'] == '2018.13.0'


def test_HDBVersion_2():
    hdb_ver = HDBVersion(context_wrap(HDB_VER_2, path=CMD_PATH))
    assert hdb_ver['branch'] == 'fa/hana2sp02'
    assert hdb_ver['compile host'] == 'ld4551'
    assert 'compile_type' not in hdb_ver
    assert hdb_ver['weekstone'] == '0000.00.0'
    assert hdb_ver.major == '2'
    assert hdb_ver.revision == '020'
    assert hdb_ver.sid == 'sr1'



"""
multipath.conf file content
===========================

MultipathConfTree - file ``/etc/multipath.conf``
------------------------------------------------

MultipathConfTreeInitramfs - command ``lsinitrd -f /etc/multipath.conf``
------------------------------------------------------------------------
"""
import string

from insights.core import ConfigParser, CommandParser
from insights.core.plugins import parser
from insights.parsr import (EOF, Forward, LeftCurly, Lift, LineEnd, Literal,
                            Many, Number, OneLineComment, PosMarker, QuotedString,
                            RightCurly, String, WS, WSChar, skip_none)
from insights.parsr.query import Entry
from insights.specs import Specs


def parse_doc(content, ctx):
    def to_entry(name, rest):
        if isinstance(rest, list):
            return Entry(name=name.value, children=rest, lineno=name.lineno, src=ctx)
        return Entry(name=name.value, attrs=[rest], lineno=name.lineno, src=ctx)

    Stmt = Forward()
    Num = Number & (WSChar | LineEnd)
    NULL = Literal("none", value=None)
    Comment = (WS >> OneLineComment("#").map(lambda x: None))
    BeginBlock = (WS >> LeftCurly << WS)
    EndBlock = (WS >> RightCurly << WS)
    Bare = String(set(string.printable) - (set(string.whitespace) | set("#{}'\"")))
    Name = WS >> PosMarker(String(string.ascii_letters + "_")) << WS
    Value = WS >> (Num | NULL | QuotedString | Bare) << WS
    EmptyString = String('"\'', min_length=2)
    Value = WS >> (Num | NULL | QuotedString | EmptyString | Bare) << WS
    Block = BeginBlock >> Many(Stmt).map(skip_none) << EndBlock
    Stanza = (Lift(to_entry) * Name * (Block | Value)) | Comment
    Stmt <= WS >> Stanza << WS
    Doc = Many(Stmt).map(skip_none)
    Top = Doc + EOF

    return Entry(children=Top(content)[0])


@parser(Specs.multipath_conf)
class MultipathConfTree(ConfigParser):
    """
    Exposes multipath configuration through the parsr query interface.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """
    def parse_doc(self, content):
        return parse_doc("\n".join(content), ctx=self)


def get_tree(root=None):
    """
    This is a helper function to get a multipath configuration component for
    your local machine or an archive. It's for use in interactive sessions.
    """
    from insights import run
    return run(MultipathConfTree, root=root).get(MultipathConfTree)


@parser(Specs.multipath_conf_initramfs)
class MultipathConfTreeInitramfs(ConfigParser, CommandParser):
    """
    Exposes the multipath configuration from initramfs image through the
    parsr query interface.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """

    bad_lines = [
        'no <initramfs file> specified and the default image'
    ]

    def __init__(self, context):
        super(MultipathConfTreeInitramfs, self).__init__(
                context, extra_bad_lines=self.bad_lines)

    def parse_doc(self, content):
        return parse_doc("\n".join(content), ctx=self)


def get_tree_from_initramfs(root=None):
    """
    This is a helper function to get a multipath configuration(from initramfs
    image) component for your local machine or an archive. It's for use in
    interactive sessions.
    """
    from insights import run
    return run(MultipathConfTreeInitramfs, root=root).get(MultipathConfTreeInitramfs)



"""
OVSappctlFdbShowBridge - command ``/usr/bin/ovs-appctl fdb/show [bridge-name]``
===============================================================================

This module provides class ``OVSappctlFdbShowBridgeCount`` to parse the
output of command ``/usr/bin/ovs-appctl fdb/show [bridge-name]``.

Sample command output::

    port VLAN  MAC Age
    6       1 MAC1 118
    3       0 MAC2 24
"""
from insights.core import CommandParser, LegacyItemAccess
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ovs_appctl_fdb_show_bridge)
class OVSappctlFdbShowBridge(CommandParser, LegacyItemAccess):
    """
       This class provides processing for the output of the command
       ``/usr/bin/ovs-appctl fdb/show [bridge-name]``.

       Sample content collected by insights-client::

           port  VLAN  MAC                Age
              6    29  aa:bb:cc:dd:ee:ff  270
              3    27  gg:hh:ii:jj:kk:ll  266
              1   100  mm:nn:oo:pp:qq:rr  263

       Sample parsed output::

           {
               'br-int': [
                             {'port': '6', 'VLAN': '29', 'MAC': 'aa:bb:cc:dd:ee:ff', 'Age': '270'},
                             {'port': '3', 'VLAN': '27', 'MAC': 'gg:hh:ii:jj:kk:ll', 'Age': '266'},
                             {'port': '1', 'VLAN': '100', 'MAC': 'mm:nn:oo:pp:qq:rr', 'Age': '263'}
                         ]
           }

       Attributes:
           data (dict): A dictionary where each element contains the bridge-name
                        as key and list of dictionary elements having MAC info as
                        value.

       Raises:
           SkipComponent: When the file is empty or data is not present for a bridge.

       Examples:

            >>> len(data["br_tun"])
            2
            >>> data.get("br_tun")[1]["MAC"] == "gg:hh:ii:jj:kk:ll"
            True
            >>> int(data["br_tun"][0]["port"])
            7
    """

    def parse_content(self, content):
        if len(content) == 0:
            raise SkipComponent("Empty file")

        self.data = {}
        # Extract the bridge name
        bridge_name = self.file_path.split("ovs-appctl_fdb.show_")[-1]

        header = content[0].split()
        self.data[bridge_name] = [dict(zip(header, entry.split(None, len(header)))) for entry in content[1:]]
        if not self.data[bridge_name]:
            raise SkipComponent("No data present for {0}".format(bridge_name))



"""
RearLocalConf - File /etc/rear/local.conf
=========================================
"""

from insights.core import TextFileOutput
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.rear_local_conf)
class RearLocalConf(TextFileOutput):
    """
    Parses content of "/etc/rear/local.conf".

    Typical content of "/etc/rear/local.conf" is::

        BACKUP_RESTORE_MOVE_AWAY_FILES=( /boot/grub/grubenv /boot/grub2/grubenv )


    Examples:
        >>> type(local_conf)
        <class 'insights.parsers.rear_conf.RearLocalConf'>
        >>> local_conf.lines[0] == 'BACKUP_RESTORE_MOVE_AWAY_FILES=( /boot/grub/grubenv /boot/grub2/grubenv )'
        True
    """

    def parse_content(self, content):
        content = get_active_lines(content)
        if not content:
            raise SkipComponent
        super(RearLocalConf, self).parse_content(content)
